/**
 * <p>Die Klasse wertet die Messages des Servers aus und ruft leitet die
 * see, hear, body_sense, init und score Nachrichten an die entsprechenden Methoden der Klasse STokenizer weiter.</p>
 * @author Johannes Wechsler, Alexander Pacnik
 * @version 1.0
 */

public class EventHandler
{
   /** Variable zur Erkennung von see Nachrichten */
   private boolean isVisualEvent;
   /** Variable zur Erkennung von body_sense Nachrichten */
   private boolean isBodyEvent;
   /** Variable zur Erkennung von hear Nachrichten */
   private boolean isAccusticEvent;
   /** Variable zur Erkennung von init Nachrichten */
   private boolean isInitEvent;
   /** Variable zur Erkennung von score Nachrichten */
   private boolean isScoreEvent;
   /** Objekt vom Typ CommunicationChannel. */
   private CommunicationChannel communicationChannel;
   /** Objekt vom Typ WorldModel. */
   private WorldModel worldModel;
   /** Objekt vom Typ STokenizer. */
   private STokenizer sTokenizer;
   /** Variable zur Speicherung der vom Server empfangenen Nachricht. */
   private String msg;

  /**
  * Legt eine <code>EventHandler</code> Instanz an.
  *
  * @param communicationChannel Die Kommunikation mit dem Server laueft hierueber
  * @param worldModel Das Weltmodell des Spielers wird hier gespeichert
  */
   public EventHandler(CommunicationChannel communicationChannel, WorldModel worldModel)
   {
      this.communicationChannel = communicationChannel;
      this.worldModel = worldModel;
      sTokenizer = new STokenizer(worldModel);
   }

   /**
   * Empfaengt Nachrichten ueber den CommunicationChannel vom Server und wertet
   * diese aus. Die see, hear, body_sense, init und score Nachrichten werden
   * an die entsprechenden Methoden in der Klasse STokenizer uebergeben.
   *  @param msg Der ankommende String.
   */
   public void event()
   {
      isVisualEvent   = false;
      isBodyEvent     = false;
      isAccusticEvent = false;
      isInitEvent     = false;
      isScoreEvent    = false;

      while ((isVisualEvent==false) && (isBodyEvent==false) && (isAccusticEvent==false  && (isInitEvent == false) && (isScoreEvent == false)))
      {
         msg = communicationChannel.receive(); // receive server message - block until one arrives

         //Body Events
         if (msg.startsWith("(sense_body"))
         {
            sTokenizer.bodyTokenizer(msg);
            isBodyEvent=true;
         }

         //See Events
         else if (msg.startsWith("(see"))
         {
            sTokenizer.visualTokenizer(msg);
            isVisualEvent=true;
         }

         //Accustic Events
         else if (msg.startsWith("(hear"))
         {
            sTokenizer.accusticTokenizer(msg);
            isAccusticEvent=true;
         }

         //Init Events
         else if (msg.startsWith("(init"))
         {
            sTokenizer.initTokenizer(msg);
            isInitEvent=true;
         }

         //Score Events
         else if (msg.startsWith("(score"))
         {
            sTokenizer.scoreTokenizer(msg);
            isScoreEvent=true;
         }
      }
   }

   /** Rueckgabe der Variable isVisualEvent. */
   public boolean isVisualEvent()
   { return isVisualEvent; }

   /** Rueckgabe der Variable isAccusticEvent. */
   public boolean isAccusticEvent()
   { return isAccusticEvent; }

   /** Rueckgabe der Variable isBodyEvent. */
   public boolean isBodyEvent()
   { return isBodyEvent; }

   /** Rueckgabe der Variable isInitEvent. */
   public boolean isInitEvent()
   {  return isInitEvent; }

   /** Rueckgabe der Variable isScoreEvent. */
   public boolean isScoreEvent()
   {  return isScoreEvent; }
}
