/**
 * <p>Die Klasse definiert einen Spieler. Jeder Spieler hat ein eigenes Weltmodell
 * und Komponenten fuer die Kommunikation mit dem Server, Verarbeitung der
 * Servernachrichten und Auswahl der Spielzuege (Strategie).</p>
 * @author Alexander Pacnik, Johannes Wechsler
 * @version 2.0
 */

import java.util.*;

public class Player extends Thread
{
   /** TeamName*/
   private String teamName;
   /** Kommunikationskomponente */
   private CommunicationChannel communicationChannel;
   /** Modul zur Verarbeitung der Servernachrichten */
   private EventHandler eventHandler;
   /** Strategiekomponente */
   private Strategie strategy;
   /** Weltmodell */
   private WorldModel worldModel;
   /** Variable gibt an, ob der Spieler der Torwart ist */
   private boolean goalie;

   /**
    * Legt eine <code>Player</code> Instanz an.
    *
    * @param servername dieser Wert gibt den Servernamen an
    * @param serverport dieser Wert gibt den Port an, auf dem der Server
    * nach ankommenden Paketen lauscht
    * @param teamName dieser Wert ist der Name unseres Teams
    */
   public Player(String teamName, int serverport, String servername, boolean goalie)
   {
      worldModel = new WorldModel();
      this.teamName = teamName;
      communicationChannel = new CommunicationChannel (servername, serverport);
      eventHandler = new EventHandler(communicationChannel, worldModel);
      strategy = new Strategie(communicationChannel, worldModel);
      this.goalie = goalie;
      worldModel.setTeamName(teamName);
   }

    /**
     * Diese Methode ueberschreibt die run() von Thread. Sie wird ausgefuehrt, wenn
     * start() fuer einen Spieler aus der Klasse RoboClient aufgerufen wird. In der
     * Methode wird gelauscht, bis eine Nachricht vom Server kommt, anschliessend
     * diese ausgewertet und die Strategie gestartet.
     */
   public void run()
   {
      //Spieler beim Server anmelden
      if(goalie)
         communicationChannel.send(new String("(init " + teamName + " (version 7.0) (goalie))"));
      else
         communicationChannel.send(new String("(init " + teamName + " (version 7.0))"));

      while (true)
      {
         //Auf Nachricht vom Server warten
         try
         { eventHandler.event(); }
         catch (Exception e)
         { System.out.println("Player: Could not start eventHandler"); }

         //Starten der Strategie bei einem BodyEvent (alle 100ms)
         if( (eventHandler.isBodyEvent() == true) || (eventHandler.isInitEvent() == true) )
         { strategy.start(); }

         //Aktualisieren der GUI
         if((worldModel.getID() == 1) && ( (eventHandler.isBodyEvent() == true) || (eventHandler.isVisualEvent() == true)  || (eventHandler.isInitEvent() == true) ) )
         { RoboClient.guiUpdate(); }

         //Aktualisieren des Spielstandes
         if((worldModel.getCurrentTime() % 50 == 0) && (worldModel.getCurrentTime() != 0))
         { communicationChannel.send(new String("(score)")); }
      }
   }

   /** Rueckgeben des Weltmodells des Spielers*/
   public WorldModel getWorldModel()
   { return this.worldModel; }

}

