/**
 * <p>Die Klasse fuehrt die Berechnungen der eigenen Position, der Blickrichtung
 * und der Laufrichtung aus. Ausserdem werden die Positionen der Mit- und Gegenspieler,
 * die geschaetzten Positionen der Flaggen sowie aktuelle und zukuenftige Position
 * des Balls bestimmt.</p>
 * @author Johannes Wechsler, Alexander Pacnik
 * @version 2.0
 */

import java.util.*;

public class SParser
{
   /** Objekt vom Typ WorldModel. */
   private WorldModel worldModel;

   /**
   * Legt eine <code>SParser</code> Instanz an.
   *
   * @param worldModel Das Weltmodell des Spielers wird hier gespeichert
   */
   public SParser(WorldModel worldModel)
   {
      this.worldModel = worldModel;
   }

   /**
   * Berechnet die eigene Laufrichtung normalisiert auf -180 bis +180 Grad
   */
   public void runDirectionCalculation()
   {
      //Berechnung der Laufrichtung aus Blickrichtung und Winkel des Kopfs
      double runDirection = worldModel.getRealViewDirection() + worldModel.getHeadAngle();

      //Normalisierung der RunDirection auf -180 bis +180 Grad
      if (runDirection > 180)
      { runDirection -= 360; }
      else if (runDirection < (-180))
      { runDirection += 360; }

      //Setzen der Werte im Weltmodell
      worldModel.setOwnPlayerRunDirection(runDirection);
   }

   /**
   * Berechnet die wahrgenommenen Positionen der 55 Flaggen
   */
   public void flagCalculation()
   {
      for(int ID=1; ID <= 55; ID++)
      {
         //Berechne die wahrgenommene Position, wenn die Flagge gesehen wird
         if((worldModel.getFlagTime(ID)) == worldModel.getCurrentTime())
         {
            //Relative Entfernung und Richtung abfragen
            double distance = worldModel.getFlagDistance(ID);
            double direction = worldModel.getFlagDirection(ID);

            double estPosX = -1000;
            double estPosY = -1000;

            //Eigene Position abfragen
            double playerPosX = worldModel.getOwnPlayerPosX();
            double playerPosY = worldModel.getOwnPlayerPosY();

            //Blickrichtung abfragen
            double viewDirection = worldModel.getOwnPlayerViewDirection();

            //Absolute Richtung, in der die Flagge gesehen wird
            double realDirection = viewDirection + direction;

            //Entfernung und Richtung in X und Y Werte umrechnen
            estPosX = distance * (Math.cos(Math.toRadians(realDirection)));
            estPosY = distance * (Math.sin(Math.toRadians(realDirection)));

            //Addition der X und Y Werte zur eigenen Position
            estPosX += playerPosX;
            estPosY += playerPosY;

            //Setzen der Werte im Weltmodell
            worldModel.setFlagEstPosX(ID, estPosX);
            worldModel.setFlagEstPosY(ID, estPosY);
         }
      }
   }

   /**
   * Berechnet die aktuelle und zukuenftigen Positionen des Balls
   */
   public void ballCalculation()
   {
      //Berechne die  Position, wenn der Ball gesehen wird
      if((worldModel.getBallTime()) == worldModel.getCurrentTime() &&
        (worldModel.getBallDistance() != 1000) && (worldModel.getBallDirection() != -1000))
      {
         //Initialsierungen
         double posX      = -1000;
         double posY      = -1000;
         double nextPosX1 = -1000;
         double nextPosY1 = -1000;
         double nextPosX2 = -1000;
         double nextPosY2 = -1000;

         //Aktualisierung der vorherigen Position des Balls im Weltmodell
         worldModel.setBallPreviousPosX(worldModel.getBallPosX());
         worldModel.setBallPreviousPosY(worldModel.getBallPosY());

         //Relative Entfernung und Richtung abfragen
         double distance = worldModel.getBallDistance();
         double direction = worldModel.getBallDirection();

         //Eigene Position abfragen
         double playerPosX = worldModel.getOwnPlayerPosX();
         double playerPosY = worldModel.getOwnPlayerPosY();

         //Blickrichtung abfragen
         double viewDirection = worldModel.getOwnPlayerViewDirection();

         //Absolute Richtung, in der der Ball gesehen wird
         double realDirection = viewDirection + direction;

         //Entfernung und Richtung in X und Y Werte umrechnen
         posX = distance * (Math.cos(Math.toRadians(realDirection)));
         posY = distance * (Math.sin(Math.toRadians(realDirection)));

         //Addition der X und Y Werte zur eigenen Position
         posX += playerPosX;
         posY += playerPosY;

         //Setzen der Werte im Weltmodell
         if (posX != -1000)
         { worldModel.setBallPosX(posX); }
         if (posY != -1000)
         { worldModel.setBallPosY(posY); }

         //Zuknftige Ballpositionen berechnen, wenn der Ball sich wirklich bewegt
         if ((Math.abs(worldModel.getBallPosX() - worldModel.getBallPreviousPosX()) >=0.5) && (worldModel.getBallPosY() - worldModel.getBallPreviousPosY() >=0.5))
         {
            //Berechnung der Differenz zwischen vorheriger und aktueller Position
            double deltaPosX = worldModel.getBallPosX() - worldModel.getBallPreviousPosX();
            double deltaPosY = worldModel.getBallPosY() - worldModel.getBallPreviousPosY();

            //Berechnung der zukuenftigen Position aus der vorherigen und aktuellen Position
            nextPosX1 = worldModel.getBallPosX() + 3 * deltaPosX;
            nextPosY1 = worldModel.getBallPosY() + 3 * deltaPosY;
            nextPosX2 = worldModel.getBallPosX() + 6 * deltaPosX;
            nextPosY2 = worldModel.getBallPosY() + 6 * deltaPosY;

            //Setzen der Werte im Weltmodell
            worldModel.setBallNextPosX1(nextPosX1);
            worldModel.setBallNextPosY1(nextPosY1);
            worldModel.setBallNextPosX2(nextPosX2);
            worldModel.setBallNextPosY2(nextPosY2);
         }
         //Zukuenftige Position gleich aktuelle Position setzen, wenn der Ball sich nicht bewegt
         else
         {
            worldModel.setBallNextPosX1(posX);
            worldModel.setBallNextPosY1(posY);
            worldModel.setBallNextPosX2(posX);
            worldModel.setBallNextPosY2(posY);
         }

         //Setzen der Werte fuer die Strategie. Bei Spielen auf der rechten Seite Spiegelung am Ursprung
         if (worldModel.getSide() == 1)
         {
            worldModel.setRealBallPosX(worldModel.getBallPosX());
            worldModel.setRealBallPosY(worldModel.getBallPosY());

            worldModel.setRealBallPreviousPosX(worldModel.getBallPreviousPosX());
            worldModel.setRealBallPreviousPosY(worldModel.getBallPreviousPosY());

            worldModel.setRealBallNextPosX1(worldModel.getBallNextPosX1());
            worldModel.setRealBallNextPosY1(worldModel.getBallNextPosY1());

            worldModel.setRealBallNextPosX2(worldModel.getBallNextPosX2());
            worldModel.setRealBallNextPosY2(worldModel.getBallNextPosY2());
         }
         else
         {
            worldModel.setRealBallPosX((-1) * worldModel.getBallPosX());
            worldModel.setRealBallPosY((-1) * worldModel.getBallPosY());

            worldModel.setRealBallPreviousPosX((-1) * worldModel.getBallPreviousPosX());
            worldModel.setRealBallPreviousPosY((-1) * worldModel.getBallPreviousPosY());

            worldModel.setRealBallNextPosX1((-1) * worldModel.getBallNextPosX1());
            worldModel.setRealBallNextPosY1((-1) * worldModel.getBallNextPosY1());

            worldModel.setRealBallNextPosX2((-1) * worldModel.getBallNextPosX2());
            worldModel.setRealBallNextPosY2((-1) * worldModel.getBallNextPosY2());
         }
      }
   }

   /**
   * Berechnet die Positionen der Mit- und Gegenspieler und speichert diese in
   * einem Vector. Gewaehrleistet die Aktualitaet der gesehenen Spieler.
   *
   * @param playerTempList Die fuer die Berechnung notwendigen Daten der Spieler sind hier gespeichert
   */
   public void playerCalculation(Vector playerTempList)
   {
      for(int i=0; i< playerTempList.size(); i++)
      {

         //Eigene Position abfragen
         double playerPosX = worldModel.getOwnPlayerPosX();
         double playerPosY = worldModel.getOwnPlayerPosY();

         //Blickrichtung abfragen
         double viewDirection = worldModel.getOwnPlayerViewDirection();

         //Initalisierung der Spielerposition
         double posX = -1000;
         double posY = -1000;

         //Variable gibt an, ob der Spieler vorhanden ist
         boolean playerActualized = false;

         //Daten des Spielers abfragen
         double  direction  = ((PlayerInfo)(playerTempList.get(i))).getDirection();
         double  distance   = ((PlayerInfo)(playerTempList.get(i))).getDistance();
         boolean goalie     = ((PlayerInfo)(playerTempList.get(i))).getGoalie();
         double  distChange = ((PlayerInfo)(playerTempList.get(i))).getDistanceChange();
         double  dirChange  = ((PlayerInfo)(playerTempList.get(i))).getDirectionChange();
         int     time       = ((PlayerInfo)(playerTempList.get(i))).getTime();
         int     ID         = ((PlayerInfo)(playerTempList.get(i))).getID();
         int     team       = ((PlayerInfo)(playerTempList.get(i))).getTeam();


         //Absolute Richtung, in der der Spieler gesehen wird
         double realDirection = viewDirection + direction;

         //Entfernung und Richtung in X und Y Werte umrechnen
         posX = distance * (Math.cos(Math.toRadians(realDirection)));
         posY = distance * (Math.sin(Math.toRadians(realDirection)));

         //Addition der X und Y Werte zur eigenen Position
         posX += playerPosX;
         posY += playerPosY;

         //berpruefen, ob der Spieler schon vorhaden ist und ggf. dessen Werte aktualisieren
         for (int j=0; j< worldModel.playerList.size(); j++)
         {
            if((worldModel.playerList.get(j) != null) && (((PlayerInfo)(worldModel.playerList.get(j))).getID() == ID) && (((PlayerInfo)(worldModel.playerList.get(j))).getTeam() == team))
            {
               ((PlayerInfo)(worldModel.playerList.get(j))).setGoalie(goalie);
               ((PlayerInfo)(worldModel.playerList.get(j))).setDistance(distance);
               ((PlayerInfo)(worldModel.playerList.get(j))).setDirection(direction);
               ((PlayerInfo)(worldModel.playerList.get(j))).setDistanceChange(distChange);
               ((PlayerInfo)(worldModel.playerList.get(j))).setDirectionChange(dirChange);

               //Setzen der Werte im Weltmodell
               if (Math.abs(posX) <= 52.5)
               {
                  ((PlayerInfo)(worldModel.playerList.get(j))).setPosX(posX);
                  //Setzen der Werte fuer die Strategie. Bei Spielen auf der rechten Seite Spiegelung am Ursprung
                  if (worldModel.getSide() == 1)
                  { ((PlayerInfo)(worldModel.playerList.get(j))).setRealPosX(posX);}
                  else
                  { ((PlayerInfo)(worldModel.playerList.get(j))).setRealPosX((-1) * posX);}
               }
               if (Math.abs(posY) <= 34)
               {
                  ((PlayerInfo)(worldModel.playerList.get(j))).setPosY(posY);
                  //Setzen der Werte fuer die Strategie. Bei Spielen auf der rechten Seite Spiegelung am Ursprung
                  if (worldModel.getSide() == 1)
                  { ((PlayerInfo)(worldModel.playerList.get(j))).setRealPosY(posY);}
                  else
                  { ((PlayerInfo)(worldModel.playerList.get(j))).setRealPosY((-1) * posY);}
               }
               ((PlayerInfo)(worldModel.playerList.get(j))).setTime(time);
               playerActualized = true;
            }
         }

         //Hinzufuegen eines neuen Spielers, wenn der Spieler noch nicht vorhanden ist
         if (playerActualized == false)
         {
            //Setzen der Werte im Weltmodell, wenn der Spieler sich im Feld befindet
            if ((Math.abs(posX) <= 52.5) && (Math.abs(posY) <= 34))
            {
               PlayerInfo player = new PlayerInfo();
               player.setTeam(team);
               player.setID(ID);
               player.setGoalie(goalie);
               player.setDistance(distance);
               player.setDirection(direction);
               player.setDistanceChange(distChange);
               player.setDirectionChange(dirChange);
               player.setPosX(posX);
               player.setPosY(posY);
               //Setzen der Werte fuer die Strategie. Bei Spielen auf der rechten Seite Spiegelung am Ursprung
               if (worldModel.getSide() == 1)
               {
                  player.setRealPosX(posX);
                  player.setRealPosY(posY);
               }
               else
               {
                  player.setRealPosX((-1) * posX);
                  player.setRealPosY((-1) * posY);
               }
               player.setTime(time);
               worldModel.playerList.add(player);
            }
         }
      }

      //Loeschen aller Spieler, die in den letzten zehn Zyklen nicht mehr gesehen wurden
      for (int k=0; k< worldModel.playerList.size(); k++)
      {
         if(worldModel.playerList.get(k) != null)
         {
            if ((((PlayerInfo)(worldModel.playerList.get(k))).getTime() +10) <= worldModel.getCurrentTime())
            { worldModel.playerList.remove(k);}
         }
      }

     //Solange den aeltesten Spieler loeschen, bis nur noch 22 Spieler gesehen werden
      while (worldModel.playerList.size() > 22)
      {
         //Index des zu loeschenden Spielers
         int deleteIndex = 0;

         //Aeltesten Spieler bestimmen
         for (int k=1; k< worldModel.playerList.size(); k++)
         {
            if ((((PlayerInfo)(worldModel.playerList.get(k))).getTime()) > (((PlayerInfo)(worldModel.playerList.get(deleteIndex))).getTime()))
            { deleteIndex = k; }
         }

         //Spieler loeschen
         worldModel.playerList.remove(deleteIndex);
      }
   }

   /**
   * Berechnet die eigene Position sowie die Blick- und die Laufrichtung, die auf
   * -180 bis +180 Grad normalisiert sind.
   */
   public void ownPlayerCalculation()
   {
      //Initialisierungen
      LineInfo lineTmp              = null;
      FlagInfo flagTmp              = null;
      FlagInfo flagTmpNear          = null;
      FlagInfo flagTmpNearest       = null;
      double   absDirection         = -1000;
      double   realDirection        = -1000;
      double   realDirectionNear    = -1000;
      double   posX                 = worldModel.getOwnPlayerPosX();
      double   posY                 = worldModel.getOwnPlayerPosY();
      int      side                 = worldModel.getSide();
      boolean  isOutside            = false;
      boolean  nearOutside          = false;

      //Naechste und zweitnaechste Fahnen bestimmen, die die aktuelle Zeit aufweisen
      for (int ID = 1; ID <= 55; ID ++)
      {
         flagTmp = worldModel.getFlag(ID);
         if ((flagTmp != null) && (worldModel.getFlagTime(ID) == worldModel.getCurrentTime()) &&
            (flagTmp.getDistance() != 1000) && (flagTmp.getDirection() != -1000))
         {
            //Wenn schon zwei Fahnen gefunden wurden, ueberpruefe, ob die aktuelle naeher ist
            if ((flagTmpNearest != null) && (flagTmpNear != null))
            {
               if (flagTmp.getDistance() < flagTmpNearest.getDistance())
               {
                  flagTmpNear = flagTmpNearest;
                  flagTmpNearest = flagTmp;
               }
               else if ((flagTmp.getDistance() >= flagTmpNearest.getDistance()) && (flagTmp.getDistance() < flagTmpNear.getDistance()))
               { flagTmpNear = flagTmp; }
            }
            //Wenn nur eine Fahne gefunden wurde, ueberpruefe, ob die aktuelle naeher ist bzw. speichere sie als zweitnaechste
            else if ((flagTmpNearest != null) && (flagTmpNear == null))
            {
               if (flagTmp.getDistance() < flagTmpNearest.getDistance())
               {
                  flagTmpNear = flagTmpNearest;
                  flagTmpNearest = flagTmp;
               }
               else if ((flagTmp.getDistance() >= flagTmpNearest.getDistance()))
               { flagTmpNear = flagTmp; }
            }
            //Wenn noch keine Fahne gefunden wurde, speichere die aktuelle als die naechste
            else if ((flagTmpNearest == null))
            { flagTmpNearest = flagTmp; }
         }
      }

      //Bestimme eine Linie, die im aktuellen Zyklus gesehen wurde
      if((worldModel.getLineTime(2) == worldModel.getCurrentTime()) && (worldModel.getLineDirection(2) != -1000) && (worldModel.getLineDistance(2) != 1000))
      { lineTmp = worldModel.getLine(2); }
      else if((worldModel.getLineTime(4) == worldModel.getCurrentTime()) && (worldModel.getLineDirection(4) != -1000) && (worldModel.getLineDistance(4) != 1000))
      { lineTmp = worldModel.getLine(4); }
      else if((worldModel.getLineTime(3) == worldModel.getCurrentTime()) && (worldModel.getLineDirection(3) != -1000) && (worldModel.getLineDistance(3) != 1000))
      { lineTmp = worldModel.getLine(3); }
      else if((worldModel.getLineTime(1) == worldModel.getCurrentTime()) && (worldModel.getLineDirection(1) != -1000) && (worldModel.getLineDistance(1) != 1000))
      { lineTmp = worldModel.getLine(1); }

      //Berechne aus dem Winkel, mit dem man auf die Linie schaut, die eigene Blickrichtung normalisiert auf -180 bis +180 Grad
      if((lineTmp != null) && (lineTmp.getTime() == worldModel.getCurrentTime()) && (lineTmp.getDirection() != -1000))
      {

         double relDirection = lineTmp.getDirection();
         int ID = lineTmp.getID();

         //Spielrichtung: von links nach rechts
         //Obere Linie
         if (ID == 1)
         {
            if(relDirection < 0)
            { absDirection = -180 + Math.abs(relDirection); }
            if(relDirection >= 0)
            { absDirection = (-1) * Math.abs(relDirection); }
         }
         //Rechte Linie
         if (ID == 2)
         {
            if (relDirection <= 0)
            { absDirection = Math.abs(relDirection) - 90; }
            if (relDirection > 0)
            { absDirection = 90 - relDirection; }
         }
         //Untere Linie
         if (ID == 3)
         {
            if(relDirection < 0)
            { absDirection = Math.abs(relDirection); }
            if(relDirection >= 0)
            { absDirection = 180 - relDirection; }
         }
         //Linke Linie
         if (ID == 4)
         {
            if (relDirection >= 0)
            { absDirection = -90 - Math.abs(relDirection); }
            if (relDirection < 0)
            { absDirection = 90 + Math.abs(relDirection); }
         }

         //Setze die fuer interne Berechnungen relevante Blickrichtung
         if (absDirection != -1000)
         { worldModel.setOwnPlayerViewDirection(absDirection); }

         //Setze die fuer die Strategie relevante Blickrichtung im Weltmodell
         //Spielrichtung: von links nach rechts
         if (side == 1)
         { worldModel.setRealViewDirection(absDirection); }
         //Spielrichtung: von rechts nach links
         else
         {
         if (absDirection >= 0)
         { worldModel.setRealViewDirection(absDirection - 180); }
         else if (absDirection < 0)
         { worldModel.setRealViewDirection(absDirection + 180); }
         }
      }
      //Positionsberechnung mit einer Linie und einer Flagge
      if ((lineTmp!= null) && (flagTmpNearest != null) && (flagTmpNearest.getDirection() != -1000) && (flagTmpNearest.getDistance() != 1000) && (lineTmp.getDirection() != -1000) && (lineTmp.getDistance() != 1000) && (flagTmpNearest.getTime() == worldModel.getCurrentTime()) && (lineTmp.getTime() == worldModel.getCurrentTime()))
      {
         //Absolute Richtung, in der die Flagge gesehen wird
         realDirection = absDirection + flagTmpNearest.getDirection();

         //Entfernung und Richtung in X und Y Werte umrechnen
         posX = flagTmpNearest.getDistance() * Math.cos(Math.toRadians(realDirection));
         posY = flagTmpNearest.getDistance() * Math.sin(Math.toRadians(realDirection));

         //Subtraktion der X und Y Werte von der Position der Flagge
         posX = flagTmpNearest.getPosX() - posX;
         posY = flagTmpNearest.getPosY() - posY;
      }

      //Aktualisieren der absoluten X und Y Position, sofern sich der Spieler im Spielfeld befindet
      if ((posX != -1000)  && (Math.abs(posX) <= 52.5))
      {
         worldModel.setOwnPlayerPosX(posX);
         //Setzen der Werte fuer die Strategie. Bei Spielen auf der rechten Seite Spiegelung am Ursprung
         if (worldModel.getSide() == 1)
         { worldModel.setRealOwnPlayerPosX(posX); }
         else
         { worldModel.setRealOwnPlayerPosX((-1) * posX); }
      }
      if ((posY != -1000)  && (Math.abs(posY) <= 34))
      {
         worldModel.setOwnPlayerPosY(posY);
         //Setzen der Werte fuer die Strategie. Bei Spielen auf der rechten Seite Spiegelung am Ursprung
         if (worldModel.getSide() == 1)
         { worldModel.setRealOwnPlayerPosY(posY); }
         else
         { worldModel.setRealOwnPlayerPosY((-1) * posY); }
      }

      //Bestimmen, ob der Spieler sich nahe am Spielfeld oder ausserhalb des Feldes befindet
      for (int ID = 1; ID <= 36; ID ++)
      {
        //Abfrage, ob der Spieler eine aussere Flagge in weniger als fuenf Metern sieht
         if ((Math.abs(worldModel.getFlag(ID).getDistance()) <= 5)  || (Math.abs(posX) >= 52.5) || (Math.abs(posY) >= 34))
         {
            isOutside  = true;
            nearOutside = true;
         }
         //Abfrage, ob der Spieler sich nahe am Spielfeldrand befindet
         else if ((Math.abs(posX + 52.5) <= 2)  || (Math.abs(posX - 52.5) <= 2)  || (Math.abs(posY + 34) <= 2)  || (Math.abs(posY - 34) <= 2))
         { nearOutside = true; }
      }

      //Setzen der Werte im Weltmodell
      worldModel.setOwnPlayerOutside(isOutside);
      worldModel.setOwnPlayerNearOutside(nearOutside);
   }
}