/**
 * <p>Die Klasse wertet die see, body_sense, init, hear und score Nachrichten
 * des Servers aus. Die Nachrichten werden geparst und im Weltmodell die
 * entsprechenden (relativen) Werte gesetzt. </p>
 * @author Johannes Wechsler, Alexander Pacnik
 * @version 2.0
 */


import java.util.*;

public class STokenizer
{
   private WorldModel worldModel;
   private SParser sParser;
   private String msg;

   /** Bekommt das Weltmodell bergeben und reicht es weiter an den SParser.
    *  @param worldModel Ist das aktuelle Weltmodell des Spielers.
    */
   public STokenizer(WorldModel worldModel)
   {
      this.worldModel = worldModel;
      sParser = new SParser(worldModel);
   }

   /** Verarbeitet die see Nachrichten.
    *  @param msg Der ankommende String.
    *  */
   public void visualTokenizer(String msg)
   {
      //TeamName
      String teamName = worldModel.getTeamName();
      StringTokenizer tokenizer = new StringTokenizer(msg, "()");

      //Speichern der gesehenen Spieler in einem Vector und bergabe dieses
      //Vecotors an die Methode playerCalculation
      Vector playerTempList = new Vector(20);

      int time = Integer.parseInt(tokenizer.nextToken().substring(4).trim());

      //Aktualisierung der Zeit
      worldModel.setCurrentTime(time);

      while(tokenizer.hasMoreTokens())                                          //solange noch ein see Objekt existiert
      {
        String code = tokenizer.nextToken();                                    // code: ObjectName

        if (!code.equals(" "))                                                  //to handle see messages without content
         {
            //values: Distance, Direction, DistChange, DirChange, BodyFacingDir, HeadFacingDir
            String values = null;
            if (tokenizer.hasMoreTokens()) {values = tokenizer.nextToken();}

            //skip next token (whitespace) if possible
            if (tokenizer.hasMoreTokens()) {tokenizer.nextToken();}

            double [] value = new double [10];
            try
            {
               //Convertion of String values to Double Values
               StringTokenizer tokenizerValues = new StringTokenizer(values, " ");

               for (int i=1; tokenizerValues.hasMoreTokens(); i++)
               { value[i] = Double.valueOf(tokenizerValues.nextToken()).doubleValue(); }
            }
            catch (NumberFormatException nfe)
            {
               value[1] = 1000;
               value[2] = -1000;
            }
            catch (NullPointerException npe)
            {
               value[1] = 1000;
               value[2] = -1000;
            }

            //Ball
            if ((code.equals("b")) || (code.equals("B")))
            {
               worldModel.setBallDistance(value[1]);
               worldModel.setBallDirection(value[2]);
               worldModel.setBallDistanceChange(value[4]);
               worldModel.setBallDirectionChange(value[4]);
               worldModel.setBallTime(time);
            }

            //Linien
            else if (code.equals("l t"))
            {
               worldModel.setLineDistance(1, value[1]);
               worldModel.setLineDirection(1, value[2]);
               worldModel.setLineTime(1, time);
            }
            else if (code.equals("l r"))
            {
               worldModel.setLineDistance(2, value[1]);
               worldModel.setLineDirection(2, value[2]);
               worldModel.setLineTime(2, time);
            }
            else if (code.equals("l b"))
            {
               worldModel.setLineDistance(3, value[1]);
               worldModel.setLineDirection(3, value[2]);
               worldModel.setLineTime(3, time);
            }
            else if (code.equals("l l"))
            {
               worldModel.setLineDistance(4, value[1]);
               worldModel.setLineDirection(4, value[2]);
               worldModel.setLineTime(4, time);
            }

            //Flags
            //Call: sParser.flagCalculation(ID, distance, direction, time)
            else if (code.equals("f t l 50"))
            {
               worldModel.setFlagDistance(1, value[1]);
               worldModel.setFlagDirection(1, value[2]);
               worldModel.setFlagTime(1, time);
            }
            else if (code.equals("f t l 40"))
            {
               worldModel.setFlagDistance(2, value[1]);
               worldModel.setFlagDirection(2, value[2]);
               worldModel.setFlagTime(2, time);
            }
            else if (code.equals("f t l 30"))
            {
               worldModel.setFlagDistance(3, value[1]);
               worldModel.setFlagDirection(3, value[2]);
               worldModel.setFlagTime(3, time);
            }
            else if (code.equals("f t l 20"))
            {
               worldModel.setFlagDistance(4, value[1]);
               worldModel.setFlagDirection(4, value[2]);
               worldModel.setFlagTime(4, time);
            }
            else if (code.equals("f t l 10"))
            {
               worldModel.setFlagDistance(5, value[1]);
               worldModel.setFlagDirection(5, value[2]);
               worldModel.setFlagTime(5, time);
            }
            else if (code.equals("f t 0"))
            {
               worldModel.setFlagDistance(6, value[1]);
               worldModel.setFlagDirection(6, value[2]);
               worldModel.setFlagTime(6, time);
            }
            else if (code.equals("f t r 10"))
            {
               worldModel.setFlagDistance(7, value[1]);
               worldModel.setFlagDirection(7, value[2]);
               worldModel.setFlagTime(7, time);
            }
            else if (code.equals("f t r 20"))
            {
               worldModel.setFlagDistance(8, value[1]);
               worldModel.setFlagDirection(8, value[2]);
               worldModel.setFlagTime(8, time);
            }
            else if (code.equals("f t r 30"))
            {
               worldModel.setFlagDistance(9, value[1]);
               worldModel.setFlagDirection(9, value[2]);
               worldModel.setFlagTime(9, time);
            }
            else if (code.equals("f t r 40"))
            {
               worldModel.setFlagDistance(10, value[1]);
               worldModel.setFlagDirection(10, value[2]);
               worldModel.setFlagTime(10, time);
            }
            else if (code.equals("f t r 50"))
            {
               worldModel.setFlagDistance(11, value[1]);
               worldModel.setFlagDirection(11, value[2]);
               worldModel.setFlagTime(11, time);
            }
            else if (code.equals("f r t 30"))
            {
               worldModel.setFlagDistance(12, value[1]);
               worldModel.setFlagDirection(12, value[2]);
               worldModel.setFlagTime(12, time);
            }
            else if (code.equals("f r t 20"))
            {
               worldModel.setFlagDistance(13, value[1]);
               worldModel.setFlagDirection(13, value[2]);
               worldModel.setFlagTime(13, time);
            }
            else if (code.equals("f r t 10"))
            {
               worldModel.setFlagDistance(14, value[1]);
               worldModel.setFlagDirection(14, value[2]);
               worldModel.setFlagTime(14, time);
            }
            else if (code.equals("f r 0"))
            {
               worldModel.setFlagDistance(15, value[1]);
               worldModel.setFlagDirection(15, value[2]);
               worldModel.setFlagTime(15, time);
            }
            else if (code.equals("f r b 10"))
            {
               worldModel.setFlagDistance(16, value[1]);
               worldModel.setFlagDirection(16, value[2]);
               worldModel.setFlagTime(16, time);
            }
            else if (code.equals("f r b 20"))
            {
               worldModel.setFlagDistance(17, value[1]);
               worldModel.setFlagDirection(17, value[2]);
               worldModel.setFlagTime(17, time);
            }
            else if (code.equals("f r b 30"))
            {
               worldModel.setFlagDistance(18, value[1]);
               worldModel.setFlagDirection(18, value[2]);
               worldModel.setFlagTime(18, time);
            }
            else if (code.equals("f b r 50"))
            {
               worldModel.setFlagDistance(19, value[1]);
               worldModel.setFlagDirection(19, value[2]);
               worldModel.setFlagTime(19, time);
            }
            else if (code.equals("f b r 40"))
            {
               worldModel.setFlagDistance(20, value[1]);
               worldModel.setFlagDirection(20, value[2]);
               worldModel.setFlagTime(20, time);
            }
            else if (code.equals("f b r 30"))
            {
               worldModel.setFlagDistance(21, value[1]);
               worldModel.setFlagDirection(21, value[2]);
               worldModel.setFlagTime(21, time);
            }
            else if (code.equals("f b r 20"))
            {
               worldModel.setFlagDistance(22, value[1]);
               worldModel.setFlagDirection(22, value[2]);
               worldModel.setFlagTime(22, time);
            }
            else if (code.equals("f b r 10"))
            {
               worldModel.setFlagDistance(23, value[1]);
               worldModel.setFlagDirection(23, value[2]);
               worldModel.setFlagTime(23, time);
            }
            else if (code.equals("f b 0"))
            {
               worldModel.setFlagDistance(24, value[1]);
               worldModel.setFlagDirection(24, value[2]);
               worldModel.setFlagTime(24, time);
            }
            else if (code.equals("f b l 10"))
            {
               worldModel.setFlagDistance(25, value[1]);
               worldModel.setFlagDirection(25, value[2]);
               worldModel.setFlagTime(25, time);
            }
            else if (code.equals("f b l 20"))
            {
               worldModel.setFlagDistance(26, value[1]);
               worldModel.setFlagDirection(26, value[2]);
               worldModel.setFlagTime(26, time);
            }
            else if (code.equals("f b l 30"))
            {
               worldModel.setFlagDistance(27, value[1]);
               worldModel.setFlagDirection(27, value[2]);
               worldModel.setFlagTime(27, time);
            }
            else if (code.equals("f b l 40"))
            {
               worldModel.setFlagDistance(28, value[1]);
               worldModel.setFlagDirection(28, value[2]);
               worldModel.setFlagTime(28, time);
            }
            else if (code.equals("f b l 50"))
            {
               worldModel.setFlagDistance(29, value[1]);
               worldModel.setFlagDirection(29, value[2]);
               worldModel.setFlagTime(29, time);
            }
            else if (code.equals("f l b 30"))
            {
               worldModel.setFlagDistance(30, value[1]);
               worldModel.setFlagDirection(30, value[2]);
               worldModel.setFlagTime(30, time);
            }
            else if (code.equals("f l b 20"))
            {
               worldModel.setFlagDistance(31, value[1]);
               worldModel.setFlagDirection(31, value[2]);
               worldModel.setFlagTime(31, time);
            }
            else if (code.equals("f l b 10"))
            {
               worldModel.setFlagDistance(32, value[1]);
               worldModel.setFlagDirection(32, value[2]);
               worldModel.setFlagTime(32, time);
            }
            else if (code.equals("f l 0"))
            {
               worldModel.setFlagDistance(33, value[1]);
               worldModel.setFlagDirection(33, value[2]);
               worldModel.setFlagTime(33, time);
            }
            else if (code.equals("f l t 10"))
            {
               worldModel.setFlagDistance(34, value[1]);
               worldModel.setFlagDirection(34, value[2]);
               worldModel.setFlagTime(34, time);
            }
            else if (code.equals("f l t 20"))
            {
               worldModel.setFlagDistance(35, value[1]);
               worldModel.setFlagDirection(35, value[2]);
               worldModel.setFlagTime(35, time);
            }
            else if (code.equals("f l t 30"))
            {
               worldModel.setFlagDistance(36, value[1]);
               worldModel.setFlagDirection(36, value[2]);
               worldModel.setFlagTime(36, time);
            }
            else if (code.equals("f l t"))
            {
               worldModel.setFlagDistance(37, value[1]);
               worldModel.setFlagDirection(37, value[2]);
               worldModel.setFlagTime(37, time);
            }
            else if (code.equals("f c t"))
            {
               worldModel.setFlagDistance(38, value[1]);
               worldModel.setFlagDirection(38, value[2]);
               worldModel.setFlagTime(38, time);
            }
            else if (code.equals("f r t"))
            {
               worldModel.setFlagDistance(39, value[1]);
               worldModel.setFlagDirection(39, value[2]);
               worldModel.setFlagTime(39, time);
            }
            else if (code.equals("f g r t"))
            {
               worldModel.setFlagDistance(40, value[1]);
               worldModel.setFlagDirection(40, value[2]);
               worldModel.setFlagTime(40, time);
            }
            else if (code.equals("f g r b"))
            {
               worldModel.setFlagDistance(42, value[1]);
               worldModel.setFlagDirection(42, value[2]);
               worldModel.setFlagTime(42, time);
            }
            else if (code.equals("f r b"))
            {
               worldModel.setFlagDistance(43, value[1]);
               worldModel.setFlagDirection(43, value[2]);
               worldModel.setFlagTime(43, time);
            }
            else if (code.equals("f c b"))
            {
               worldModel.setFlagDistance(44, value[1]);
               worldModel.setFlagDirection(44, value[2]);
               worldModel.setFlagTime(44, time);
            }
            else if (code.equals("f l b"))
            {
               worldModel.setFlagDistance(45, value[1]);
               worldModel.setFlagDirection(45, value[2]);
               worldModel.setFlagTime(45, time);
            }
            else if (code.equals("f g l b"))
            {
               worldModel.setFlagDistance(46, value[1]);
               worldModel.setFlagDirection(46, value[2]);
               worldModel.setFlagTime(46, time);
            }
            else if (code.equals("f g l t"))
            {
               worldModel.setFlagDistance(48, value[1]);
               worldModel.setFlagDirection(48, value[2]);
               worldModel.setFlagTime(48, time);
            }
            else if (code.equals("f p r t"))
            {
               worldModel.setFlagDistance(49, value[1]);
               worldModel.setFlagDirection(49, value[2]);
               worldModel.setFlagTime(49, time);
            }
            else if (code.equals("f p r c"))
            {
               worldModel.setFlagDistance(50, value[1]);
               worldModel.setFlagDirection(50, value[2]);
               worldModel.setFlagTime(50, time);
            }
            else if (code.equals("f p r b"))
            {
               worldModel.setFlagDistance(51, value[1]);
               worldModel.setFlagDirection(51, value[2]);
               worldModel.setFlagTime(51, time);
            }
            else if (code.equals("f p l b"))
            {
               worldModel.setFlagDistance(52, value[1]);
               worldModel.setFlagDirection(52, value[2]);
               worldModel.setFlagTime(52, time);
            }
            else if (code.equals("f p l c"))
            {
               worldModel.setFlagDistance(53, value[1]);
               worldModel.setFlagDirection(53, value[2]);
               worldModel.setFlagTime(53, time);
            }
            else if (code.equals("f p l t"))
            {
               worldModel.setFlagDistance(54, value[1]);
               worldModel.setFlagDirection(54, value[2]);
               worldModel.setFlagTime(54, time);
            }
            else if (code.equals("f r t"))
            {
               worldModel.setFlagDistance(55, value[1]);
               worldModel.setFlagDirection(55, value[2]);
               worldModel.setFlagTime(55, time);
            }

            //Goals
            //Call: sParser.flagCalculation(ID, distance, direction, time)
            else if (code.equals("g l"))
             {
               worldModel.setFlagDistance(47, value[1]);
               worldModel.setFlagDirection(47, value[2]);
               worldModel.setFlagTime(47, time);
            }
            else if (code.equals("g r"))
            {
               worldModel.setFlagDistance(41, value[1]);
               worldModel.setFlagDirection(41, value[2]);
               worldModel.setFlagTime(41, time);
            }

            //Objekte, die ausserhalb des Sichtfelds, aber innerhalb von 3 Metern um den Spieler sich befinden,
            //werden ohne Namen erkannt: vgl manual page 30

            //Mitspieler und Gegenspieler
            else if ((code.startsWith("p")) || (code.equals("P")))
            {
               StringTokenizer tokenizerPlayer = new StringTokenizer(code, " ");
               String [] playerStrings = new String [10];

               for (int i=0; tokenizerPlayer.hasMoreTokens(); i++)
               { playerStrings[i] = tokenizerPlayer.nextToken(); }

               int team       = -1000;
               int ID         = -1000;
               boolean goalie = false;

               if (playerStrings[1] != null)                                    //team: our = 1; opponent = 2;
               {
                  if (playerStrings[1].equals("\"" +teamName +"\""))
                  { team = 1;}
                  else if (playerStrings[1] != null)
                  { team = 2;}
                  else
                  { team = -1000;}
               }

               //ID: if available, otherwise -1000
               if (playerStrings[2] != null)
               { ID = Integer.parseInt(playerStrings[2]); }

               //goalie: true if goalie, othewise false
               if ((playerStrings[3] != null) && (playerStrings[3].equals("goalie")))
               { goalie = true;}

               //Neuen Spieler mit den Werten vom Server anlegen und dem Vector
               //hinzufgen. Vector dann am Ende der Methode playerCalculation()
               //im SParser bergeben
               PlayerInfo playerTemp = new PlayerInfo();
               playerTemp.setTeam(team);
               playerTemp.setID(ID);
               playerTemp.setGoalie(goalie);
               playerTemp.setTime(time);
               playerTemp.setDistance(value[1]);
               playerTemp.setDirection(value[2]);
               playerTemp.setDistanceChange(value[3]);
               playerTemp.setDirectionChange(value[4]);
               playerTempList.add(playerTemp);
            }
         }
      }
      //Nachdem die Server Nachrichten ausgewertet sind, finden jetzt die
      //eigentlichen Berechnungen statt

      sParser.ownPlayerCalculation();
      sParser.ballCalculation();
      sParser.playerCalculation(playerTempList);
      //Berechne die Flaggenpositionen nur, wenn diese in der GUI auch gezeichnet werden
      if (worldModel.getGuiPaintsFlags() == true)
      { sParser.flagCalculation(); }
   }

   /** Verarbeitet die init Nachrichten.
    *  @param msg Der ankommende String.
    *  */
   public void initTokenizer(String msg)
   {
      StringTokenizer tokenizer = new StringTokenizer(msg, "()");               //Klammern entfernen
      String code = tokenizer.nextToken();

      StringTokenizer tokenizer2 = new StringTokenizer(code, " ");              //Leerzeichen entfernen
      String [] values = new String[4];

      //Initialisierung
      int i     = 0;
      int side  = -1000;
      int ID    = -1000;

      //Strings abspeichern
      while(tokenizer2.hasMoreTokens())
      {
         values[i] = tokenizer2.nextToken();
         i++;
      }

      //Seite
      if(values[1].equals("l"))
      { side = 1; }
      else
      { side = 2; }

      //Nummer
      ID = Integer.parseInt(values[2]);

      //playMode = 0 (beforeKickOff)

      worldModel.setSide(side);
      worldModel.setPlayMode(0);
      worldModel.setID(ID);
   }

   /** Verarbeitet die body_sense Nachrichten.
    *  @param msg Der ankommende String.
    *  */
   public void bodyTokenizer(String msg)
   {
      StringTokenizer tokenizer = new StringTokenizer(msg, "()");
      int time = Integer.parseInt(tokenizer.nextToken().substring(11).trim());

      String [] values = new String [24];
      int i=0;

      while(tokenizer.hasMoreTokens())
      {
        values[i] = tokenizer.nextToken();
        i++;
      }

      int    viewModeQuality  =0;                                               //high=1 oder low=2
      int    viewModeRange    =0;                                               //narrow=1, normal=2 oder wide=3
      double stamina          =0;
      double effort           =0;
      double amountOfSpeed    =0;
      double directionOfSpeed =0;
      double headAngle        =0;
      int    kickCount        =0;
      int    dashCount        =0;
      int    turnCount        =0;
      int    sayCount         =0;
      int    turnNeckCount    =0;
      int    catchCount       =0;
      int    moveCount        =0;
      int    changeViewCount  =0;

      //viewModeQuality and viewModeRange
      if((values[0] != null) && (values[0].equals("view_mode high normal")))
      {
         viewModeQuality = 1;
         viewModeRange = 2;
      }

      else if(values[0].equals("view_mode high narrow "))
      {
         viewModeQuality = 1;
         viewModeRange = 1;
      }

      else if(values[0].equals("view_mode low narrow"))
      {
         viewModeQuality = 2;
         viewModeRange = 1;
      }

      else if(values[0].equals("view_mode low normal"))
      {
         viewModeQuality = 2;
         viewModeRange = 2;
      }

      else if(values[0].equals("view_mode high wide"))
      {
         viewModeQuality = 1;
         viewModeRange = 3;
      }

      else if(values[0].equals("view_mode low wide"))
      {
         viewModeQuality = 2;
         viewModeRange = 3;
      }

      //stamina and effort
      StringTokenizer tokenizer2 = new StringTokenizer(values[2], " ");
      tokenizer2.nextToken();
      stamina = Double.parseDouble(tokenizer2.nextToken());
      effort = Double.parseDouble(tokenizer2.nextToken());

      //amountOfSpeed and directionOfSpeed
      StringTokenizer tokenizer4 = new StringTokenizer(values[4], " ");
      tokenizer4.nextToken();
      amountOfSpeed = Double.parseDouble(tokenizer4.nextToken());
      directionOfSpeed = Double.parseDouble(tokenizer4.nextToken());

      //headAngle
      StringTokenizer tokenizer6 = new StringTokenizer(values[6], " ");
      String headAngleString = tokenizer6.nextToken();
      headAngle = Double.parseDouble(tokenizer6.nextToken());

      //kickCount
      StringTokenizer tokenizer8 = new StringTokenizer(values[8], " ");
      tokenizer8.nextToken();
      kickCount = Integer.parseInt(tokenizer8.nextToken());

      //dashCount
      StringTokenizer tokenizer10 = new StringTokenizer(values[10], " ");
      tokenizer10.nextToken();
      dashCount = Integer.parseInt(tokenizer10.nextToken());

      //turnCount
      StringTokenizer tokenizer12 = new StringTokenizer(values[12], " ");
      tokenizer12.nextToken();
      turnCount = Integer.parseInt(tokenizer12.nextToken());

      //sayCount
      StringTokenizer tokenizer14 = new StringTokenizer(values[14], " ");
      tokenizer14.nextToken();
      sayCount = Integer.parseInt(tokenizer14.nextToken());

      //turnNeckCount
      StringTokenizer tokenizer16 = new StringTokenizer(values[16], " ");
      tokenizer16.nextToken();
      turnNeckCount = Integer.parseInt(tokenizer16.nextToken());

      //catchCount
      StringTokenizer tokenizer18 = new StringTokenizer(values[18], " ");
      tokenizer18.nextToken();
      catchCount = Integer.parseInt(tokenizer18.nextToken());

      //moveCount
      StringTokenizer tokenizer20 = new StringTokenizer(values[20], " ");
      tokenizer20.nextToken();
      moveCount = Integer.parseInt(tokenizer20.nextToken());

      //changeViewCountString
      StringTokenizer tokenizer22 = new StringTokenizer(values[22], " ");
      tokenizer22.nextToken();
      changeViewCount = Integer.parseInt(tokenizer22.nextToken());

      //Aktualisieren der Werte im Weltmodell
      worldModel.setViewModeQuality(viewModeQuality);
      worldModel.setViewModeRange(viewModeRange);
      worldModel.setStamina(stamina);
      worldModel.setEffort(effort);
      worldModel.setAmountOfSpeed(amountOfSpeed);
      worldModel.setDirectionOfSpeed(directionOfSpeed);
      worldModel.setHeadAngle(headAngle);
      worldModel.setKickCount(kickCount);
      worldModel.setDashCount(dashCount);
      worldModel.setTurnCount(turnCount);
      worldModel.setSayCount(sayCount);
      worldModel.setTurnNeckCount(turnNeckCount);
      worldModel.setCatchCount(catchCount);
      worldModel.setMoveCount(moveCount);
      worldModel.setChangeViewCount(changeViewCount);
      worldModel.setCurrentTime(time);

      sParser.runDirectionCalculation();
   }

   /** Verarbeitet die hear Nachrichten.
    *  @param msg Der ankommende String.
    *  */
   public void accusticTokenizer(String msg)
   {
      //Klammern entfernen
      StringTokenizer tokenizer = new StringTokenizer(msg, "()");
      String code = tokenizer.nextToken();

      //Leerzeichen entfernen
      StringTokenizer tokenizer2 = new StringTokenizer(code, " ");
      String [] values = new String[6];

      //Initialisierung
      int i                 = 0;
      int time              = -1000;
      double ballPosX       = -1000;
      double ballPosY       = -1000;
      int playMode          = -1000;
      String sender         = null;
      String hearMessage    = null;
      String valueString    = null;

      //Strings abspeichern
      while((tokenizer2.hasMoreTokens()) && (i<6))
      {
         values[i] = tokenizer2.nextToken();   // code: ObjectName
         i++;
      }

      if ((values[1] != null))
      { time = Integer.parseInt(values[1]); }

      sender = values[2];

      //messages vom referee
      if ((sender != null) && (sender.equals("referee")))
      {
         if (values[3] != null)
         {
            //playModeString = values[3];
            //Seite 46
            if (values[3].startsWith("goal_l"))
            { playMode = 13; }
            else if (values[3].startsWith("goal_r"))
            { playMode = 14; }
            else if ((values[3].startsWith("foul_l")) || (values[3].equals("goalie_catch_ball_l")))
            { playMode = 8; }
            else if ((values[3].startsWith("foul_r")) || (values[3].equals("goalie_catch_ball_r")))
            { playMode = 7; }
            else if (values[3].startsWith("time_up"))
            { playMode = 4; }
            else if (values[3].equals("half_time"))
            { playMode = 15; }
            //Seite 45
            else if ((values[3].equals("before_kick_off")) || (values[3].equals("time_extended")))
            { playMode = 0; }
            else if (values[3].equals("play_on"))
            { playMode = 3; }
            else if (values[3].equals("time_over"))
            { playMode = 4; }
            else if (values[3].equals("kick_off_l"))
            { playMode = 1; }
            else if (values[3].equals("kick_off_r"))
            { playMode = 2; }
            else if (values[3].equals("kick_in_l"))
            { playMode = 5; }
            else if (values[3].equals("kick_in_r"))
            { playMode = 6; }
            else if (values[3].equals("corner_kick_l"))
            { playMode = 11; }
            else if (values[3].equals("corner_kick_r"))
            { playMode = 12; }
            else if (values[3].equals("goal_kick_l"))
            { playMode = 9; }
            else if (values[3].equals("goal_kick_r"))
            { playMode = 10; }
            else if (values[3].equals("drop_ball"))
            { playMode = 3; }
            else if (values[3].equals("offside_l"))
            { playMode = 7; }
            else if (values[3].equals("offside_r"))
            { playMode = 8; }
         }
      }

      //messages von Mitspielern
      if ((values[3] != null) && (values[3].equals("BALLPOSITION")))
      {
         if ((values[4] != null))
         { ballPosX = Double.parseDouble(values[4]); }
         if ((values[5] != null))
         { ballPosY = Double.parseDouble(values[5]); }
      }

      //Aktualisiere currentTime und abhaengig vom Sender die PlayModes
      //oder ballPosX, ballPosY und ballTime
      if (sender.equals("referee") && (playMode != -1000))
      {
         worldModel.setCurrentTime(time);
         worldModel.setPlayMode(playMode);
      }
      else if ((ballPosX != -1000) && (ballPosX != -1000))
         {
            worldModel.setBallTime(time);
            worldModel.setCurrentTime(time);
            worldModel.setBallPosX(ballPosX);
            worldModel.setBallPosY(ballPosY);
         }
         else
         { worldModel.setCurrentTime(time); }
      sParser.runDirectionCalculation();
   }

   /** Verarbeitet die score Nachrichten.
    *  @param msg Der ankommende String.
    */
   public void scoreTokenizer(String msg)
   {
      //Klammern entfernen
      StringTokenizer tokenizer = new StringTokenizer(msg, "()");
      String code = tokenizer.nextToken();

      //Leerzeichen entfernen
      StringTokenizer tokenizer2 = new StringTokenizer(code, " ");
      String [] values = new String[4];

      //Initialisierung
      int i          = 0;
      int time       = -1000;
      int ourScore   = 0;
      int theirScore = 0;

      //Strings abspeichern
      while(tokenizer2.hasMoreTokens())
      {
         values[i] = tokenizer2.nextToken();   // code: ObjectName
         i++;
      }

      if ((values[1] != null))   //time
      { time = Integer.parseInt(values[1]); }
      if ((values[2] != null))   //ourScore
      { ourScore   = Integer.parseInt(values[2]); }
      if ((values[3] != null))   //theirScore
      { theirScore = Integer.parseInt(values[3]); }

      worldModel.setCurrentTime(time);
      worldModel.setOurScore(ourScore);
      worldModel.setTheirScore(theirScore);
   }
}
