import java.lang.Math.*;
import java.util.*;

/*******************************************************************************
* <p>Title: Tactical Moves</p>
* <p>Description: In dieser Klasse sind alle taktischen Aktionen und Abfragen
*  definiert, die von unserer Strategie verwendet werden </p>
* <p>Copyright: Copyright (c) 2002</p>
* @author Bjrn Konrad, Holger Stephan, Christoph Zwiener
* @version 1.0
*******************************************************************************/

public class TacticalMoves
{

   WorldModel wmodel;
   CommunicationChannel comch;

   public TacticalMoves(CommunicationChannel communicationChannel, WorldModel wmodel)
   {
      this.comch = communicationChannel;
      this.wmodel = wmodel;
   }


   /****************************************************************************
   *   public void moveGoalie()
   *
   *   Bewegt(move) den Goalie hat zu einer Abstoposition in Anhngigkeit
   *   seiner derzeitigen Position
   *   @author Holger Stephan
   *   @version 1.0
   ****************************************************************************/

   public void moveGoalie()
   {
      if(wmodel.getRealOwnPlayerPosY() < 0)
         move(-37,19);
      else
         move(-37,-19);
   }

   /****************************************************************************
   *   public void move(double x, doulbe y)
   *
   *   Bewegt den Spieler zur angegebenen Position
   *   @author Holger Stephan
   *   @version 1.0
   *   @param x X-Position
   *   @param y Y-Position
   ****************************************************************************/

   public void move(double x, double y)
   {
      comch.send(new String("(move " + x + " " + y + ")"));
   }

  /*****************************************************************************
  *   public void turn(double moment)
  *
  *   Dreht den Spieler um den Wert moment (-180 bis 180)
  *	@author Holger Stephan
  *   @version 1.0
  *   @param moment Drehwinkel
  *****************************************************************************/

   public void turn(double moment)
   {
      if(moment >= -180 && moment <= 180)
      {
	      comch.send(new String("(turn " + moment + " )"));
         wmodel.turnDirection(moment);   // Winkel an Weltmodell bermitteln
      }
   }


   /****************************************************************************
   *   public void dash(double power)
   *
   *   Beschleunigt den Spieler um power (-100 bis 100) in Richtung seines
   *	 Krpers
   *   @author Holger Stephan
   *   @version 1.0
   *   @param power Strke der Beschleunigung
   ****************************************************************************/

   public void dash(double power)
   {
      if(power >= -100 && power <= 100)
         comch.send(new String("(dash " + power + " )"));
   }


   /****************************************************************************
   *   public void kick(double power, double dir)
   *
   *   Schiesst den Ball mit power (-100 bis 100) und dem Winkel dir
   *   (-180 bis 180)
   *   @author Holger Stephan
   *   @version 1.0
   *   @param power Strke des Schusses
   *   @param dir Richtung des Schusses
   ****************************************************************************/

   public void kick(double power, double dir)
   {
      if(power >= -100 && power <= 100 && dir >= -180 && dir <= 180)
	      comch.send(new String("(kick " + power + " " + dir + " )"));
   }



   /****************************************************************************
   *   public void catchBall(double dir)
   *
   *   Diese Methode ist nur fr den Goalie. Er fngt den Ball in einem Winkel
   *   "dir", welcher relativ zu seinem Krper ist.
   *   @author Holger Stephan
   *   @version 1.0
   *   @param dir Fangwinkel
   ****************************************************************************/

   public void catchBall(double dir)
   {
      if(!wmodel.getCatchBan())		//Abfrage, ob seid dem letzten catch 5 Zyklen vergangen sind
      {
         if(dir >= -180 && dir <= 180)
         {
        	   comch.send(new String("(catch " + dir + ")"));
			   wmodel.setCatchBan(wmodel.getCurrentTime());
         }
      }
      else
      {
         goGetBall();
      }
   }

   /****************************************************************************
   *   public void turnNeck(double angle)
   *
   *   Diese Methode dreht den Kopf des Spielers um den Winkel "angle"  .
   *   Der absolute Kopfwinkel des Spielers darf sich nur um +-90 vom Krper-
   *   winkel unterscheiden
   *   @author Holger Stephan
   *   @version 1.0
   *   @param angle Drehwinkel
   ****************************************************************************/

   public void turnNeck( double angle)
   {
      double headangle = wmodel.getHeadAngle();

      if ((headangle + angle) <= 90 && (headangle + angle) >= -90) //ist eine Drehung um diesen Winkel mglich?
      {
         comch.send(new String("(turn_neck " + angle + " )"));
      }
      else
         if ((headangle + angle) > 90)
            comch.send(new String("(turn_neck " + (90-headangle) + " )")); //Drehung bis 90
         else
            comch.send(new String("(turn_neck " + (headangle-90) + " )"));	//Drehung bis -90
   }

   /****************************************************************************
   *   public void goGetBall()
   *
   *   Diese Methode bewegt den Spieler zu einer vorherberechneten Ballposition
   *   @author Holger Stephan
   *   @revised by
   *   @version 1.0
   ****************************************************************************/

   public void goGetBall()
   {
      //double abfangwinkel = wmodel.getBallDirection();
      double power;
      double abfangwinkel = calculateTurnAngle(wmodel.getRealBallPosX(),wmodel.getRealBallPosY());
      //Berechnung des Abfangwinkels

      if(wmodel.getBallDistance()>10)
      {
         abfangwinkel = calculateTurnAngle(wmodel.getRealBallNextPosX2(),wmodel.getRealBallNextPosY2());
         if ((abfangwinkel < 15) && (abfangwinkel > -15))
         {
            dash(100);
         }
         else
         {
            if(abfangwinkel > 45)
               abfangwinkel = 45;
            if(abfangwinkel < -45)
               abfangwinkel = -45;
            turn(abfangwinkel);
         }
      }
      else if(wmodel.getBallDistance()>5)
      {
         abfangwinkel = calculateTurnAngle(wmodel.getRealBallNextPosX1(),wmodel.getRealBallNextPosY1());
         if ((abfangwinkel < 15) && (abfangwinkel > -15))
         {

            dash(100);
         }
         else
         {
            if(abfangwinkel > 45)
               abfangwinkel = 45;
            if(abfangwinkel < -45)
               abfangwinkel = -45;
            turn(abfangwinkel);

         }
      }
      else if ((abfangwinkel < 15) && (abfangwinkel > -15))
      {
	      //Berechnung der Geschwindigkeit getBallDistance()
         if(wmodel.getBallDistance() >4)
            power = 100;
         else
            power = 60;
            dash(power);
      }
      else
      {
         if(abfangwinkel > 45)
            abfangwinkel = 45;
         if(abfangwinkel < -45)
            abfangwinkel = -45;
         turn(abfangwinkel);
      }
   }

   /****************************************************************************
   *   public boolean ballInPenaltyArea()
   *
   *   Prft ob der Ball in der penaltyarea ist
   *   @author Holger Stephan
   *   @return true = ja, false = nein
   *   @version 1.0
   ****************************************************************************/

   public boolean ballInPenaltyArea()
   {
         if(wmodel.getRealBallPosX() < -36 && wmodel.getRealBallPosY() < 20 && wmodel.getRealBallPosY() > -20 )
            return true;
         else
            return false;
   }

   /****************************************************************************
   *   public boolean opponentInPenaltyarea()
   *
   *   Prft ob ein Gegenspieler in der penaltyarea ist
   *   @author Holger Stephan
   *   @return true = ja, false = nein
   *   @version 1.0
   ****************************************************************************/

   public boolean opponentInPenaltyarea()
   {
      boolean player = false;
      if(wmodel.getSide() == -1000)
         return false; //-1000 = nicht def.

      for (int j=0; j< wmodel.playerList.size(); j++)
      {
         if(((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 2)
         {
            if( ((PlayerInfo)(wmodel.playerList.get(j))).getRealPosX() <= -36 && ((PlayerInfo)(wmodel.playerList.get(j))).getRealPosY() <= 20 && ((PlayerInfo)(wmodel.playerList.get(j))).getRealPosY() >= -20)
               player = true;
         }
      }
   return player;
   }

   /****************************************************************************
   *   public boolean ownPlayerNotInPenaltyarea()
   *
   *   Prft ob kein Mitspieler in der Penaltyarea ist
   *   @author Holger Stephan
   *   @return true = ja, false = nein
   *   @version 1.0
   ****************************************************************************/


   public boolean ownPlayerNotInPenaltyarea()
   {
      boolean player = true;
      if(wmodel.getSide() == -1000)
         return false; //-1000 = nicht def.
      for (int j=0; j< wmodel.playerList.size(); j++)
         {
            if(((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 1)
            {
               if( ((PlayerInfo)(wmodel.playerList.get(j))).getRealPosX() <= -36 && ((PlayerInfo)(wmodel.playerList.get(j))).getRealPosY() <= 20 && ((PlayerInfo)(wmodel.playerList.get(j))).getRealPosY() >= -20)
                  player = false;
            }
         }
   return player;
   }


   /****************************************************************************
   *   public boolean seeBall()
   *
   *   Prft, ob der Spieler den Ball in den letzten 10 Zyklen gesehen hat
   *   @author Holger Stephan
   *   @return true = ja, false = nein
   *   @version 1.0
   ****************************************************************************/

   public boolean seeBall()
   {
      if (wmodel.getBallTime()+10 < wmodel.getCurrentTime())
         return false;
      else
         return true;
   }


   /****************************************************************************
   *   public boolean nearBall()
   *
   *   Prft, ob der Spieler einer der 2 Nhesten am Ball ist
   *   @author Holger Stephan
   *   @return true = ja, false = nein
   *   @version 1.0
   ****************************************************************************/

   public boolean nearBall()
   {
      int playerNearer = 0;
      double kath1, kath2, distance;
      for (int j=0; j< wmodel.playerList.size(); j++)
      {
         if(((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 1)
         {
            kath1 = ((PlayerInfo)(wmodel.playerList.get(j))).getRealPosX() - wmodel.getRealBallPosX();
            kath2 = ((PlayerInfo)(wmodel.playerList.get(j))).getRealPosY() - wmodel.getRealBallPosY();
            distance = (kath1 * kath1) + (kath2 * kath2);
            distance = Math.sqrt(distance);
            if(distance < wmodel.getBallDistance())
               playerNearer++;
         }
      }
      if (playerNearer >= 2)
         return false;
      else
         return true;
   }

   /****************************************************************************
   *   public boolean nearestBall()
   *
   *   Prft, ob der Spieler der Nheste am Ball ist
   *   @author Holger Stephan
   *   @return true = ja, false = nein
   *   @version 1.0
   ****************************************************************************/

   public boolean nearestBall()
   {
      int playerNearer = 0;
      double kath1, kath2, distance;
      for (int j=0; j< wmodel.playerList.size(); j++)
      {
         if(((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 1)
         {
            kath1 = ((PlayerInfo)(wmodel.playerList.get(j))).getRealPosX() - wmodel.getRealBallPosX();
            kath2 = ((PlayerInfo)(wmodel.playerList.get(j))).getRealPosY() - wmodel.getRealBallPosY();
            distance = (kath1 * kath1) + (kath2 * kath2);
            distance = Math.sqrt(distance);
            if(distance < wmodel.getBallDistance())
               playerNearer++;
         }
      }
      if (playerNearer >= 1)
         return false;
      else
         return true;
   }


   /****************************************************************************
   *   public boolean ballCatchable()
   *
   *   Prft ob der Goalie den Ball fangen kann
   *   @author Holger Stephan
   *   @return true = ja, false = nein
   *   @version 1.0
   ****************************************************************************/

   public boolean ballCatchable()
   {
      if(wmodel.getBallDistance() < 2 && wmodel.getBallDirection() < 45 && wmodel.getBallDirection() > -45 )
         return true;
      else
         return false;
  }


   /****************************************************************************
   *   public void goToDefensePos()
   *
   *   Bewegt den Spieler zu seiner Defense Position
   *   @author Bjrn Konrad
   *   @version 1.0
   ****************************************************************************/

   public void goToDefensePos()
   {
      double defX = 0;
      double defY = 0;
      switch (wmodel.getNumber())
      {
         case 2: defX = -27; defY = -25;break;
         case 3: defX = -27; defY = -15;break;
         case 4: defX = -23; defY = 0 ;break;
         case 5: defX = -27; defY = 15 ;break;
         case 6: defX = -27; defY = 25 ;break;
         case 7: defX = -1; defY = -15 ;break;
         case 8: defX = -1; defY = 0 ;break;
         case 9: defX = -1; defY = 15 ;break;
         case 10: defX = -1; defY = -28 ;break;
         case 11: defX = -1; defY = 28; break;
      }
      double ballangle = this.calculateTurnAngle(wmodel.getRealBallPosX(),wmodel.getRealBallPosY());
      double angle = calculateTurnAngle(defX,defY);
      double distance = this.calculateDistance(wmodel.getRealOwnPlayerPosX(),wmodel.getRealOwnPlayerPosY(),defX,defY);
      if (distance > 3) //falls Entfernung zum Zielpunkt gro
      {
         if (Math.abs(angle) < 15)//falls Winkel zum Zielpunkt stimmig,lauf los
         {
            dash(50);
         }
         else if (Math.abs(angle) >= 15) //falls Winkel nicht stimmig, dreh dich zum Zielpunkt
         {
            if (Math.abs(angle) > 90)
            {
               if (angle < 0) turn(-90);
               else turn (90);
            }
            else turn(angle);
         }
      }
      else //falls Entfernung zum Zielpunkt klein,bleib stehen und dreh dich zum Ball
      {
         if (Math.abs(ballangle) > 15)
         {
            if (Math.abs(ballangle) > 90)
            {
               if (ballangle < 0) turn(-90);
               else turn (90);
            }
            else turn(ballangle);
         }
      }
   }

   /****************************************************************************
   *   public void goToAttackPos()
   *
   *   Bewegt den Spieler zu seiner Attack Position
   *   @author Bjrn Konrad
   *   @version 1.0
   ****************************************************************************/

   public void goToAttackPos()
   {
      double attackX = 0;
      double attackY = 0;
      switch (wmodel.getNumber())
      {
         case 2: attackX = -10; attackY = -25;break;
         case 3: attackX = -10; attackY = -15;break;
         case 4: attackX = -10; attackY = 0 ;break;
         case 5: attackX = -10; attackY = 15; break;
         case 6: attackX = -10; attackY = 25;  break;
         case 7: attackX = 15; attackY = -15;break;
         case 8: attackX = 25; attackY = 0; break;
         case 9: attackX = 15;  attackY = 15;break;
         case 10: attackX = 25; attackY = -25;  break;
         case 11: attackX = 25; attackY = 25; break;
      }
      double view = wmodel.getRealViewDirection();
      double ballangle = this.calculateTurnAngle(wmodel.getRealBallPosX(),wmodel.getRealBallPosY());
      double angle = calculateTurnAngle(attackX,attackY);
      double distance = this.calculateDistance(wmodel.getRealOwnPlayerPosX(),wmodel.getRealOwnPlayerPosY(),attackX,attackY);
      if (distance > 3) //falls Entfernung zum Zielpunkt gro
      {
         if (Math.abs(angle) < 15) //falls Winkel zum Zielpunkt stimmig,lauf los
         {
            dash(50);
         }
         else if (Math.abs(angle) >= 15) //falls Winkel nicht stimmig, dreh dich zum Zielpunkt
         {
            if (Math.abs(angle) > 90)
            {
               if (angle < 0) turn(-90);
               else turn (90);
            }
            else turn(angle);
         }
      }
      else //falls Entfernung zum Zielpunkt klein,bleib stehen und dreh dich zum Ball
      {
         if (Math.abs(ballangle) > 15)
         {
            if (Math.abs(ballangle) > 90)
            {
               if (ballangle < 0) turn(-90);
               else turn (90);
            }
            else turn(ballangle);
         }
      }
   }




   /****************************************************************************
   *   public void goToNormalPos()
   *
   *   Bewegt den Spieler zu seiner Normal Position
   *   @author Bjrn Konrad
   *   @version 1.0
   ****************************************************************************/

   public void goToNormalPos()
   {
      double normX = 0;
      double normY = 0;
      switch (wmodel.getNumber())
      {
         case 1: normX = -40; normY = 0;break;
         case 2: normX = -23; normY = -25;break;
         case 3: normX = -23; normY = -15;break;
         case 4: normX = -20; normY = 0 ;break;
         case 5: normX = -23; normY = 15; break;
         case 6: normX = -23; normY = 25;  break;
         case 7: normX = 5; normY = -23;break;
         case 8: normX = 15; normY = 0; break;
         case 9: normX = 5;  normY = 23;break;
         case 10: normX = 15; normY = -25;  break;
         case 11: normX = 15; normY = 25; break;
      }
      double ballangle = this.calculateTurnAngle(wmodel.getRealBallPosX(),wmodel.getRealBallPosY());
      double angle = calculateTurnAngle(normX,normY);
      double distance = this.calculateDistance(wmodel.getRealOwnPlayerPosX(),wmodel.getRealOwnPlayerPosY(),normX,normY);
      if (distance > 3) //falls Entfernung zum Zielpunkt gro
      {
         if (Math.abs(angle) < 15) //falls Winkel zum Zielpunkt stimmig,lauf los
         {
            dash(50);
         }
         else if (Math.abs(angle) >= 15) //falls Winkel nicht stimmig, dreh dich zum Zielpunkt
         {
            if (Math.abs(angle) > 90)
            {
               if (angle < 0) turn(-90);
               else turn (90);
            }
            else turn(angle);
         }
      }
      else //falls Entfernung zum Zielpunkt klein,bleib stehen und dreh dich zum Ball
      {
         if (Math.abs(ballangle) > 15)
         {
            if (Math.abs(ballangle) > 90)
            {
               if (ballangle < 0) turn(-90);
               else turn (90);
            }
            else turn(ballangle);
         }
      }
   }

   /****************************************************************************
   *   public void calculateTurnAngle()
   *
   *   Berechnet den Winkel, um den sich der Spieler drehen muss, damit er zu
   *   einer bestimmten Position kommt
   *   @return= Gibt Winkel zurck
   *   @param posX XPosition
   *   @param posY YPostion
   *   @author Bjrn Konrad
   *   @version 1.0
   ****************************************************************************/

   public double calculateTurnAngle(double posX,double posY)
   {
      double AngleGamma;
      double AngleTurn = 0;
      double x = wmodel.getRealOwnPlayerPosX();
      double y = wmodel.getRealOwnPlayerPosY();
      double DefensePosX = posX;
      double DefensePosY = posY;
      double AngleAlpha = wmodel.getRealViewDirection();
      double a = (DefensePosY - y);
      double b = (DefensePosX - x);
      double c = a/b;
      double AngleBeta = Math.atan(c);
      AngleBeta = Math.abs(AngleBeta * 180 / Math.PI);
      if (x > DefensePosX && y > DefensePosY)   //Quadrant 1
      {
         if (AngleAlpha < 0)  //Spieler schaut nach rechts
         {
            AngleAlpha = Math.abs(AngleAlpha);
            if (AngleAlpha < (180 - AngleBeta))   //Spieler kann sich nach rechts drehen
            {
               AngleTurn = (180 - AngleAlpha - AngleBeta) * (-1);
            }
            else if (AngleAlpha > (180 - AngleBeta))    //Spieler muss sich nach links drehen
            {
               AngleTurn = (-180 + AngleAlpha + AngleBeta);
            }
            else if (AngleAlpha == AngleBeta)
            {
               AngleTurn = 0;
            }
         } //Ende Spieler schaut nach rechts
      else if (AngleAlpha >= 0)
      {
         if (AngleAlpha < AngleBeta)
         {
            AngleTurn = (180 + AngleAlpha - AngleBeta) * (-1);
         }
         else if (AngleAlpha > AngleBeta)
         {
            AngleTurn = (180 + AngleBeta - AngleAlpha);
         }
      }
   } //Ende Quadrant 1

   //----------------------Quadrant 2  Beginn-----------------------------------
   else if (x > DefensePosX && y < DefensePosY)    //Quadrant 2
   {
      if (AngleAlpha < 0)  //Spieler schaut nach rechts
      {
         AngleAlpha = Math.abs(AngleAlpha);
         if (AngleAlpha < AngleBeta)
         {
            AngleTurn = 180 - AngleBeta + AngleAlpha;
         }
         else if (AngleAlpha > AngleBeta)
         {
            AngleTurn = (180 + AngleBeta - AngleAlpha) * (-1);
         }
      }
      else if (AngleAlpha >= 0)   //Spieler schaut nach links
      {
         if (AngleAlpha < 180 - AngleBeta)   //Spieler kann sich nach rechts drehen
         {
            AngleTurn = 180 - AngleBeta - AngleAlpha;
         }
         else if (AngleAlpha > 180 - AngleBeta)    //Spieler muss sich nach links drehen
         {
            AngleTurn = (-180 + AngleBeta + AngleAlpha) * (-1);
         }
         else if (AngleAlpha == AngleBeta)
         {
            AngleTurn = 180;
         }
      }
   } //Ende Quadrant 2

   //----------------------Quadrant 3  Beginn-----------------------------------
   if (x < DefensePosX && y > DefensePosY)
   {
      if (AngleAlpha < 0)  //Spieler schaut nach links
      {
         AngleAlpha = Math.abs(AngleAlpha);
         if (AngleAlpha < AngleBeta)   //Spieler kann sich nach rechts drehen
         {
            AngleTurn = (AngleBeta - AngleAlpha) * (-1);
         }
         else if (AngleAlpha > AngleBeta)    //Spieler muss sich nach links drehen
         {
            AngleTurn = (AngleAlpha - AngleBeta);
         }
         else if (AngleAlpha == AngleBeta)
         {
            AngleTurn = 0;
         }
      } //Ende Spieler schaut nach rechts
      else if (AngleAlpha >= 0)
      {
         if (AngleAlpha < 180 - AngleBeta)
         {
            AngleTurn = -1 * (AngleBeta + AngleAlpha);
         }
         else if (AngleAlpha > 180 - AngleBeta)
         {
            AngleTurn = 180 - AngleAlpha + 180 - AngleBeta;
         }
      }
   } //Ende Quadrant 3

   //----------------------Quadrant 4  Beginn-----------------------------------
   if (x < DefensePosX && y < DefensePosY)
   {
      if (AngleAlpha > 0)
      {
         if (AngleAlpha < AngleBeta)
         {
            AngleTurn = (AngleBeta - AngleAlpha);
         }
         else if (AngleAlpha > AngleBeta)
         {
            AngleTurn = (AngleAlpha - AngleBeta) * (-1);
         }
         else if (AngleAlpha == AngleBeta)
         {
            AngleTurn = 0;
         }
      }
      else if (AngleAlpha <= 0)
      {
         AngleAlpha = Math.abs(AngleAlpha);
         if (AngleAlpha < (180 - AngleBeta))
         {
            AngleTurn = AngleBeta + AngleAlpha;
         }
         else if (AngleAlpha > (180 - AngleBeta))
         {
            AngleTurn = ((180 - AngleAlpha) + (180 - AngleBeta)) * (-1);
         }
      }
   } //Ende Quadrant 4
   return (AngleTurn);
   } //Ende Funktion

   /****************************************************************************
   *   public void setPosition()
   *
   *   Setzt den Spieler mit der Rckennummer "unum" zu Anfang des Spiels
   *   auf seine Ausgangsposition bzw. DefensePos				 				 														*
   *   @author Christoph Zwiener
   *   @version 1.0
   ****************************************************************************/

   public void setPosition()
   {
      int unum = -1;
      unum = wmodel.getNumber();
      switch (unum)
      {
         case 1: move(-50,0); break;
         case 2: move(-35,-25); break;
         case 3: move(-35,-12.5); break;
         case 4: move(-35,0); break;
         case 5: move(-35,12.5); break;
         case 6: move(-35,25); break;
         case 7: move(-15,-15); break;
         case 8: move(-15,0); break;
         case 9: move(-15,15); break;
         case 10: move(-5,-25); break;
         case 11: move(-5,25); break;
         case -1: break;   //default
      }  // switch
   }  // void setPosition

  /*****************************************************************************
  *   public void kickBallToPoint(double posX,double posY)
  *
  *   Methode lt den Spieler einen Schu ausfhren, der mglichst genau auf
  *   einen bestimmten Punkt auf dem Feld landen soll
  *   @author Christoph Zwiener
  *   @version 1.0
  *   @param posX - x-Zielkoordinate
  *   @param posY - y-Zielkoordinate
  *****************************************************************************/

   public void kickBallToPoint(double posX , double posY)
   {
      double distance = 0.0;
      int power;
      // Berechne zunchst die Entfernung zum anvisierten Punkt
      distance = calculateDistance(wmodel.getRealOwnPlayerPosX(),wmodel.getRealOwnPlayerPosY(),posX,posY);

      // abhngig von der Entfernung des anvisierten Punktes, wird die Strke,
      // mit der der Schu ausgefhrt werden soll, gewhlt & der Schu ausgefhrt
      if (distance >= 0 && distance < 5) kick(15, calculateTurnAngle(posX, posY));
      else
         if (distance >= 5 && distance < 10) kick(30, calculateTurnAngle(posX, posY));
         else
            if (distance >= 10 && distance < 15) kick(45, calculateTurnAngle(posX, posY));
            else
               if (distance >= 15 && distance < 20) kick(60, calculateTurnAngle(posX, posY));
               else
                  if (distance >= 20 && distance < 30) kick(85, calculateTurnAngle(posX, posY));
                  else
                     kick(100, calculateTurnAngle(posX, posY)); // (distance >= 30)


   } // void kickBallToPoint

   /****************************************************************************
   *   public void shootToGoal()
   *
   *   Veranlasst den Spieler sofort auf das gegnerische Tor zu schieen.
   *   Es wird, sofern der gegnerische Goalie gesehen wird, auf die Hlfte des
   *   Tors geschossen, in der der Goalie nicht steht.
   *   @author Christoph Zwiener
   *   @version 1.0
   ****************************************************************************/

   public void shootToGoal()
   {
      for (int j=0; j< wmodel.playerList.size(); j++)
      {
         if( ((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 2)
         {
            if( ((PlayerInfo)(wmodel.playerList.get(j))).getGoalie() )
            {
               if( ((PlayerInfo)(wmodel.playerList.get(j))).getRealPosY() < 0 )
                  kick(100, calculateTurnAngle(54, 4));   // Goalie steht in der linken Torhlfte
               else
                  kick(100, calculateTurnAngle(54, -4));   // Goalie steht in der rechten Torhlfte
            }
         }
      }
      kick(100, calculateTurnAngle(54, 0));   // wenn Goalie nicht gesehen wird, nimm die Mitte
   } // void shootToGoal()

   /****************************************************************************
   *   public boolean passPossible()
   *
   *   berprft ob Passen mglich ist; d.h., ob ein Mitspieler so positioniert
   *   steht, da es wahrscheinlich ist, da ein Pass zu ihm auch ankommt
   *   @author Christoph Zwiener
   *   @version 1.0
   *   @return passposs - true/false: je nach dem, ob ein Pass mglich ist oder nicht
   ****************************************************************************/

   public boolean passPossible()
   {
      boolean passposs = false;  /* Es wird zunchst davon ausgegangen, dass kein Pass mglich ist */
      boolean noOpponent = true;  // Diese Variablen sollen den Fall abfangen, in dem
      boolean noTeamMate = true;  // gar kein Gegner bzw. Mitspieler gesehen wird
      double AktTeamMateDist, AktTeamMateDir, AktOpponentDist, AktOpponentDir;
      for (int j=0; j< wmodel.playerList.size(); j++)
      {
         if( ((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 1)   //TeamMate(Mitspieler)
         {
            AktTeamMateDist = ((PlayerInfo)(wmodel.playerList.get(j))).getDistance();
            AktTeamMateDir = ((PlayerInfo)(wmodel.playerList.get(j))).getDirection();
            noTeamMate = false;
            if ( ((PlayerInfo)(wmodel.playerList.get(j))).getRealPosX() > wmodel.getRealOwnPlayerPosX() )
            {
               for (int i=0; i< wmodel.playerList.size(); i++)
               {
                  if( ((PlayerInfo)(wmodel.playerList.get(i))).getTeam() == 2) //Opponent
                  {
                     AktOpponentDist = ((PlayerInfo)(wmodel.playerList.get(i))).getDistance();
                     AktOpponentDir = ((PlayerInfo)(wmodel.playerList.get(i))).getDirection();
                     noOpponent = false;
                     if (  ((AktOpponentDir > (AktTeamMateDir+15)) || (AktOpponentDir < (AktTeamMateDir-15))) || (AktOpponentDist>AktTeamMateDist*1.3) )
                        passposs = true;
                  }
               }
            }
         }
      }
      if(noOpponent)
         passposs = true;
      if(noTeamMate)
         passposs = false;
      if (checkOffside())
         return false;
      else
         return passposs;
   }   //passPossible()


  /******************************************************************************
  *   public void passBall()
  *
  *   Methode ermittelt den Mitspieler, der am gnstigsten fr ein Passspiel
  *   positioniert steht, und spielt diesen an
  *
  *   @author Christoph Zwiener
  *   @revised by
  *   @version 1.0
  *******************************************************************************/

   public void passBall()
   {
      double AktTeamMatePosX, AktTeamMatePosY,
             OwnPlayerPosY = wmodel.getRealOwnPlayerPosY(),
             OwnPlayerPosX = wmodel.getRealOwnPlayerPosX(),
             AktTeamMateDist, AktTeamMateDir, AktOpponentDist, AktOpponentDir;
      double PassPosX = -1000;
      double PassPosY = -1000;
      boolean noOpponent = true;  // fr den Fall, dass gar kein Gegner "da ist"
      for (int j=0; j< wmodel.playerList.size(); j++)
      {
         if( ((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 1)   //TeamMate
         {
            AktTeamMateDist = ((PlayerInfo)(wmodel.playerList.get(j))).getDistance();
            AktTeamMateDir = ((PlayerInfo)(wmodel.playerList.get(j))).getDirection();
            AktTeamMatePosX = ((PlayerInfo)(wmodel.playerList.get(j))).getRealPosX();
            AktTeamMatePosY = ((PlayerInfo)(wmodel.playerList.get(j))).getRealPosY();
            for (int i=0; i< wmodel.playerList.size(); i++)
            {
               if( ((PlayerInfo)(wmodel.playerList.get(i))).getTeam() == 2) //Opponent
               {
                  AktOpponentDist = ((PlayerInfo)(wmodel.playerList.get(i))).getDistance();
                  AktOpponentDir = ((PlayerInfo)(wmodel.playerList.get(i))).getDirection();
                  noOpponent = false;
                  if (  ((AktOpponentDir > (AktTeamMateDir+15)) || (AktOpponentDir < (AktTeamMateDir-15))) || (AktOpponentDist>AktTeamMateDist*1.3) &&  AktTeamMatePosX > wmodel.getRealOwnPlayerPosX()  )
                  //dieser Mitspieler knnte also angespielt werden
                  {
                     //falls das der erste Mitspieler ist, der anspielbar ist
                     if (PassPosX == -1000 && PassPosX > OwnPlayerPosX)
                     {
                        PassPosX = AktTeamMatePosX;
                        PassPosY = AktTeamMatePosY;
                     }
                     //falls es mehrere anspielbare Mitspieler sind, entscheide Dich fr den, der am
                     //weitesten vorne steht

                     if ( (PassPosX != -1000) && (AktTeamMatePosX > PassPosX)  && (PassPosX > OwnPlayerPosX))
                     {
                        PassPosX = AktTeamMatePosX;
                        PassPosY = AktTeamMatePosY;
                     }
                  }
               }
            }
            // falls kein Gegner strt, spiele zu dem Mitspieler, dem am weitesten vorne steht
            if (noOpponent)
            {
               for (int k=0; k< wmodel.playerList.size(); k++)
               {
                  if( ((PlayerInfo)(wmodel.playerList.get(k))).getTeam() == 1)   //TeamMate
                  {
                     if ( ((PlayerInfo)(wmodel.playerList.get(k))).getRealPosX() > PassPosX  && PassPosX > OwnPlayerPosX)
                     {
                        PassPosX = AktTeamMatePosX;
                        PassPosY = AktTeamMatePosY;
                     }
                  }
               }
            } // (noOpponent)
         }
      }
      if (PassPosX != -1000 && wmodel.getRealOwnPlayerPosX() > -25) kickBallToPoint(PassPosX, PassPosY); //Pass ausfhren
      else
         if (PassPosX == -1000 && wmodel.getRealOwnPlayerPosX() > -25) kick(8, -wmodel.getOwnPlayerRunDirection());
         else if (wmodel.getRealOwnPlayerPosX() < -25)
            if (wmodel.getRealOwnPlayerPosY() < 0) kickBallToPoint(25,-25);
               else kickBallToPoint(25,25);
  } // void passBall()

   /****************************************************************************
   *   public boolean dribblePossible()
   *
   *   Untersucht die Umgebung des Spielers daraufhin, ob es eine Richtung gibt,
   *   in der sich keine Gegner befinden, sodass in diese Richtung losgedribbelt
   *   werden kann
   *   @author Christoph Zwiener
   *   @version 1.0
   *   @return dribbleposs - true or false - je nach dem ob dribbeln in eine
   *   bestimmte Richtung mglich ist
   ****************************************************************************/

   public boolean dribblePossible()
   {
      boolean dribbleposs = true; // zunchst wird davon ausgegangen, dass dribbeln mglich ist
      boolean dribbleposs12h = true;
      boolean dribbleposs2h = true; //Uhrzeiten (z.B.12h) sind bzgl. der ViewDirection des
      boolean dribbleposs4h = true; //zu verstehen (ViewDirection ^= 12h)
      boolean dribbleposs6h = true;
      boolean dribbleposs8h = true;
      boolean dribbleposs10h = true;
      double AktDir;

      //+-0 Grad: 12h ***********************************************************
      AktDir = wmodel.getRealViewDirection();
      for (int j=0; j< wmodel.playerList.size(); j++)
      {
         if( ((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 2)   //Opponent
         {
            if (  ((((PlayerInfo)(wmodel.playerList.get(j))).getDirection()+wmodel.getRealViewDirection() < (AktDir+30)) && (((PlayerInfo)(wmodel.playerList.get(j))).getDirection()+wmodel.getRealViewDirection() > (AktDir-30))) && (((PlayerInfo)(wmodel.playerList.get(j))).getDistance()<15) )
               dribbleposs12h = false;
         }
      }

      //+60 Grad: 2h ************************************************************
      AktDir = wmodel.getRealViewDirection()+60;
      for (int j=0; j< wmodel.playerList.size(); j++)
      {
         if( ((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 2)   //Opponent
         {
            if (  ((((PlayerInfo)(wmodel.playerList.get(j))).getDirection()+wmodel.getRealViewDirection() < (AktDir+30)) && (((PlayerInfo)(wmodel.playerList.get(j))).getDirection()+wmodel.getRealViewDirection() > (AktDir-30))) && (((PlayerInfo)(wmodel.playerList.get(j))).getDistance()<15) )
               dribbleposs2h = false;
         }
      }

      //+120 Grad: 4h ***********************************************************
      AktDir = wmodel.getRealViewDirection()+120;
      for (int j=0; j< wmodel.playerList.size(); j++)
      {
         if( ((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 2)   //Opponent
         {
            if (  ((((PlayerInfo)(wmodel.playerList.get(j))).getDirection()+wmodel.getRealViewDirection() < (AktDir+30)) && (((PlayerInfo)(wmodel.playerList.get(j))).getDirection()+wmodel.getRealViewDirection() > (AktDir-30))) && (((PlayerInfo)(wmodel.playerList.get(j))).getDistance()<15) )
               dribbleposs4h = false;
         }
      }

      //+180 Grad: 6h ***********************************************************
      AktDir = wmodel.getRealViewDirection()+180;
      for (int j=0; j< wmodel.playerList.size(); j++)
      {
         if( ((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 2)   //Opponent
         {
            if (  ((((PlayerInfo)(wmodel.playerList.get(j))).getDirection()+wmodel.getRealViewDirection() < (AktDir+30)) && (((PlayerInfo)(wmodel.playerList.get(j))).getDirection()+wmodel.getRealViewDirection() > (AktDir-30))) && (((PlayerInfo)(wmodel.playerList.get(j))).getDistance()<15) )
               dribbleposs6h = false;
         }
      }

      //-120 Grad: 8h ***********************************************************
      AktDir = wmodel.getRealViewDirection()-120;
      for (int j=0; j< wmodel.playerList.size(); j++)
      {
         if( ((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 2)   //Opponent
         {
            if (  ((((PlayerInfo)(wmodel.playerList.get(j))).getDirection()+wmodel.getRealViewDirection() < (AktDir+30)) && (((PlayerInfo)(wmodel.playerList.get(j))).getDirection()+wmodel.getRealViewDirection() > (AktDir-30))) && (((PlayerInfo)(wmodel.playerList.get(j))).getDistance()<15) )
                  dribbleposs8h = false;
         }
      }

      //-60 Grad: 10h ***********************************************************
      AktDir = wmodel.getRealViewDirection()-60;
      for (int j=0; j< wmodel.playerList.size(); j++)
      {
         if( ((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 2)   //Opponent
         {
            if (  ((((PlayerInfo)(wmodel.playerList.get(j))).getDirection()+wmodel.getRealViewDirection() < (AktDir+30)) && (((PlayerInfo)(wmodel.playerList.get(j))).getDirection()+wmodel.getRealViewDirection() > (AktDir-30))) && (((PlayerInfo)(wmodel.playerList.get(j))).getDistance()<15) )
               dribbleposs10h = false;
         }
      }

      if ( dribbleposs12h || dribbleposs2h || dribbleposs4h || dribbleposs6h || dribbleposs8h || dribbleposs10h ) dribbleposs = true;
      return dribbleposs;
   }

   /****************************************************************************
   *   public void dribbleBall()
   *
   *   Der Spieler orientiert sich zunchst in 6 Richtungen, die insgesamt die
   *   360 Grad um ihn herum abdecken. Dann entscheidet er sich fr die
   *   offensivste Dribbelrichtung
   *   @author Christoph Zwiener
   *   @version 1.0
   ****************************************************************************/

   public void dribbleBall()
   {
      double AktDir, TurnAngle;
      double AbsDir12h = -1000;
      double AbsDir2h = -1000;
      double AbsDir4h = -1000;
      double AbsDir6h = -1000;
      double AbsDir8h = -1000;
      double AbsDir10h = -1000;
      double DribbleDir = 360;
      boolean noOpponent = true;

      //+-0 Grad: 12h ***********************************************************
      AktDir = wmodel.getRealViewDirection();
      for (int j=0; j< wmodel.playerList.size(); j++)
      {
         if( ((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 2)   //Opponent
         {
            noOpponent = false;
            //schauen, ob der Gegner in diesem Kegel steht
            if (  ((((PlayerInfo)(wmodel.playerList.get(j))).getDirection() < (AktDir+30)) && (((PlayerInfo)(wmodel.playerList.get(j))).getDirection() > (AktDir-30))) && (((PlayerInfo)(wmodel.playerList.get(j))).getDistance()<15) )
            {
            } //in diese Richtung soll also nicht gedribbelt werden
            else   //in diese Richtung kann gedribbelt werden
            {
               AbsDir12h = wmodel.getRealViewDirection() + 0;
            }
         }
      }
      //der Spieler soll so sehr wie mglich in Richtung gegnerische Hlfte dribbeln
      if (AbsDir12h != -1000 && Math.abs(AbsDir12h) < Math.abs(DribbleDir)) DribbleDir = AbsDir12h;

      //+60 Grad: 2h ************************************************************
      AktDir = wmodel.getRealViewDirection()+60;
      for (int j=0; j< wmodel.playerList.size(); j++)
      {
         if( ((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 2)   //Opponent
         {
            noOpponent = false;
            //schauen, ob der Gegner in diesem Kegel steht
            if (  ((((PlayerInfo)(wmodel.playerList.get(j))).getDirection() < (AktDir+30)) && (((PlayerInfo)(wmodel.playerList.get(j))).getDirection() > (AktDir-30))) && (((PlayerInfo)(wmodel.playerList.get(j))).getDistance()<15) )
            {} //in diese Richtung soll also nicht gedribbelt werden
            else   //in diese Richtung kann gedribbelt werden
            {
               AbsDir2h = wmodel.getRealViewDirection() + 60;
            }
         }
      }
      //der Spieler soll so sehr wie mglich in Richtung gegnerische Hlfte dribbeln
      if (AbsDir2h != -1000 && Math.abs(AbsDir2h) < Math.abs(DribbleDir)) DribbleDir = AbsDir2h;

      //-60 Grad: 10h ***********************************************************
      AktDir = wmodel.getRealViewDirection()-60;
      for (int j=0; j< wmodel.playerList.size(); j++)
      {
         if( ((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 2)   //Opponent
         {
            noOpponent = false;
            //schauen, ob der Gegner in diesem Kegel steht
            if (  ((((PlayerInfo)(wmodel.playerList.get(j))).getDirection() < (AktDir+30)) && (((PlayerInfo)(wmodel.playerList.get(j))).getDirection() > (AktDir-30))) && (((PlayerInfo)(wmodel.playerList.get(j))).getDistance()<15) )
            {
            } //in diese Richtung soll also nicht gedribbelt werden
            else   //in diese Richtung kann gedribbelt werden
            {
               AbsDir10h = wmodel.getRealViewDirection() - 60;
            }
         }
      }
      //der Spieler soll so sehr wie mglich in Richtung gegnerische Hlfte dribbeln
      if (AbsDir10h != -1000 && Math.abs(AbsDir10h) < Math.abs(DribbleDir)) DribbleDir = AbsDir10h;

      //   ENDE DER ERMITTLUNG DER DribbleDir (Richtung, in die der Spieler dribbeln soll)

      // Umrechnung bei evtl. Unterschied zwischen ViewDirection und BodyDirection
      if ( wmodel.getOwnPlayerRunDirection() >= DribbleDir )
         TurnAngle = (Math.abs(wmodel.getOwnPlayerRunDirection()-DribbleDir))*(-1);
      else
         TurnAngle = Math.abs(wmodel.getOwnPlayerRunDirection()-DribbleDir);
      if (wmodel.getRealOwnPlayerPosY() < -20) TurnAngle = TurnAngle+5;
      if (wmodel.getRealOwnPlayerPosY() > 20) TurnAngle = TurnAngle-5;
      if (noOpponent) kick(10, calculateTurnAngle(52,0));
      else kick(10, TurnAngle);

   } // void dribbleBall()

   /****************************************************************************
   *   public void changeFieldLine()
   *
   *   Nimmt einen Flankenwechsel vor
   *   @author Bjrn Konrad
   *   @version 1.0
   ****************************************************************************/

   public void changeFieldLine()
   {
      //double PosX = wmodel.getRealOwnPlayerPosX();
      double PosY = wmodel.getRealOwnPlayerPosY();
      double shootPosX = 25;
      double shootPosY = 0;
      if (PosY >= 0)
      {
         shootPosY = -28;
      }
      else
      {
         shootPosY = 28;
      }

      this.kickBallToPoint(shootPosX,shootPosY);
   } //void changeFieldLine()

   /****************************************************************************
   *   public void checkOffside()
   *
   *   Prft, ob eine Abseitsstellung der Mitspieler vorliegt.
   *   Abseits liegt vor, falls das Maximum aller X-Positionen der Gegenspieler
   *   kleiner ist als das Maximum der X-Positionen der Mitspieler
   *   @return: offside=true oder false
   *   @author Bjrn Konrad
   *   @version 1.0
   ****************************************************************************/

   public boolean checkOffside()
   {
      double maxOpponentPosX =0;
      double maxOwnPosX =0;
      boolean ownhalf = false;

      //Durchlaufe Liste aller Spieler
      for (int j=0; j< wmodel.playerList.size(); j++)
      {
         //Mitspieler
         if(((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 1)
         {
            if (((PlayerInfo)(wmodel.playerList.get(j))).getRealPosX() > maxOwnPosX)
            {
               maxOwnPosX = ((PlayerInfo)(wmodel.playerList.get(j))).getRealPosX();
            }
         }
         //Gegenspieler
         if(((PlayerInfo)(wmodel.playerList.get(j))).getTeam() == 2)
         {
            if (((PlayerInfo)(wmodel.playerList.get(j))).getRealPosX() > maxOpponentPosX)
            {
               maxOpponentPosX = ((PlayerInfo)(wmodel.playerList.get(j))).getRealPosX();
            }

         }
      }
      if (maxOwnPosX > 0) return (maxOwnPosX > maxOpponentPosX);
      else return false;
   } //void boolean checkOffside()

   /****************************************************************************
   *   public void sayBallPos()
   *
   *   bermittelt die aktuelle Ballposition ans Weltmodell durch den Say Befehl
   *   @author Bjrn Konrad
   *   @version 1.0
   ****************************************************************************/


   public void sayBallPos()
   {
      if ((wmodel.getRealBallPosX() != 0) && (wmodel.getRealBallPosY() != 0)) //ist noch falsch!!
      {
         comch.send(new String("(say " + "BALLPOSITION " +wmodel.getRealBallPosX() + " " + wmodel.getRealBallPosY() + " )"));
      }
   } //public void sayBallPos()


   /****************************************************************************
   *   public boolean ballInOpponentArea()
   *
   *   Prft, ob der Ball in der OpponentArea ist.
   *   @author Holger Stephan
   *   @return true = ja, false = nein
   *   @version 1.0
   ****************************************************************************/

   public boolean ballInOpponentArea()
   {
      if(wmodel.getRealBallPosX() >= 15)
         return true;
      else
         return false;
   }

   /****************************************************************************
   *   public boolean ballInMiddleArea()
   *
   *   Prft, ob der Ball in der MiddleArea ist.
   *   @author Holger Stephan
   *   @return true = ja, false = nein
   *   @version 1.0
   ****************************************************************************/

   public boolean ballInMiddleArea()
   {
      if(wmodel.getRealBallPosX() >= -15 && wmodel.getRealBallPosX() <= 15)
         return true;
      else
         return false;
   }

   /****************************************************************************
   *   public boolean ballInForwardArea()
   *
   *   Prft, ob der Ball in der ForwardArea ist.
   *   @author Holger Stephan
   *   @return true = ja, false = nein
   *   @version 1.0
   ****************************************************************************/

   public boolean ballInForwardArea()
   {
      if(wmodel.getRealBallPosX() >= 25)
         return true;
      else
         return false;
   }

   /****************************************************************************
   *   public boolean ballInForwardMiddleArea()
   *
   *   Prft, ob der Ball in der ForwardMiddleArea ist.
   *   @author Holger Stephan
   *   @return true = ja, false = nein
   *   @version 1.0
   ****************************************************************************/

   public boolean ballInForwardMiddleArea()
   {
      if(wmodel.getRealBallPosX() >= 0 && wmodel.getRealBallPosX() < 25)
         return true;
      else
         return false;

   }

   /****************************************************************************
   *   public boolean ballInGoalArea()
   *
   *   Prft, ob der Ball in der GoalArea ist.
   *   @author Holger Stephan
   *   @return true = ja, false = nein
   *   @version 1.0
   ****************************************************************************/

   public boolean ballInGoalArea()
   {
      //Auf welcher Seite ist unser Tor?
      if(wmodel.getSide() == -1000)
         return false; //-1000 = nicht def.
      if(wmodel.getRealBallPosX() > 36 && wmodel.getRealBallPosY() < 20 && wmodel.getRealBallPosY() > -20 )
         return true;
      else
         return false;
   }

   /****************************************************************************
   *   public double calculateDistance(double posX1,double posY1,double posX2,double posY2)
   *
   *   Methode berechnet die Entfernung zweier Punkte in einem karthesischen
   *   Koordinatensystem	- mit Hilfe des Herrn Pythagoras - und gibt sie in
   *   Form eines double-Wertes zurck
   *   @author Christoph Zwiener
   *   @version 1.0
   *   @param posX1 - x-Koordinate des ersten Punktes
   *   @param posY1 - y-Koordinate des ersten Punktes
   *   @param posX2 - x-Koordinate des zweiten Punktes
   *   @param posY2 - y-Koordinate des zweiten Punktes
   *   @return distance - das Ergebnis der Distanzberechnung wird zurckgegeben
   ****************************************************************************/

   public double calculateDistance(double posX1,double posY1,double posX2,double posY2)
   {
      double distance, kath1, kath2 = 0.0;
      // Berechne Entfernung
      kath1 = posX1 - posX2;
      kath2 = posY1 - posY2;
      distance = (kath1 * kath1) + (kath2 * kath2);
      distance = Math.sqrt(distance);
      return distance;
   }

   /****************************************************************************
   *   public double nearToNormalPos()
   *
   *   Prft, ob sich der Spieler bereits "ungefhr" auf seiner NormalPosition
   *   befindet
   *   @author Bjrn Konrad, Holger Stephan, Christoph Zwiener
   *   @version 1.0
   *   @return boolean
   ****************************************************************************/

   public boolean nearToNormalPos()
   {
      double normX = 0;
      double normY = 0;
      switch (wmodel.getNumber())
      {
         case 1: normX = -45; normY = 0;break;
         case 2: normX = -23; normY = -30;break;
         case 3: normX = -23; normY = -15;break;
         case 4: normX = -23; normY = 15 ;break;
         case 5: normX = -23; normY = 30; break;
         case 6: normX = -20; normY = 0;  break;
         case 7: normX = 5; normY = -23;break;
         case 8: normX = 5; normY = 23; break;
         case 9: normX = 15;  normY = -28;break;
         case 10: normX = 15; normY = 0;  break;
         case 11: normX = 15; normY = 28; break;
      }
      if ( calculateDistance(wmodel.getRealOwnPlayerPosX(), wmodel.getRealOwnPlayerPosY(), normX, normY) < 5 )
         return true;
      else
         return false;
   } // nearToNormalPos()

   /****************************************************************************
   *   public double goToGoaliePos()
   *
   *   Bewegt den Torwart zu einer, in Abhngigkeit zur Ballposition berechneten,
   *   Position
   *   @author Bjrn Konrad
   *   @version 1.0
   ****************************************************************************/

   public void goToGoaliePos()
   {
      double ballposY = wmodel.getRealBallNextPosY2();
      double ballangle = Math.atan(wmodel.getRealBallPosY()/(wmodel.getRealBallPosX()-50));
      ballangle = Math.abs(ballangle * 180 / Math.PI);
      if (wmodel.isOwnPlayerOutside() == false)
         goToPos(-44, ballposY / 5);
      else
         runIntoField();
   }

   /****************************************************************************
   *   public void gotoPos()
   *
   *   Bewegt den Spieler zu einer bestimmten Position
   *   @author Bjrn Konrad
   *   @param posX X Position des Zielpunktes
   *   @param posY Y Position des Zielpunktes
   *   @version 1.0
   ****************************************************************************/

   public void goToPos(double posX,double posY)
   {
      double view = wmodel.getRealViewDirection();
      double ballangle = this.calculateTurnAngle(wmodel.getRealBallPosX(),wmodel.getRealBallPosY());
      double angle = calculateTurnAngle(posX,posY);
      double distance = this.calculateDistance(wmodel.getRealOwnPlayerPosX(),wmodel.getRealOwnPlayerPosY(),posX,posY);
      if (distance > 3)
      {
         if (Math.abs(angle) < 20)
         {
            dash(30);
         }
         else if (Math.abs(angle) >= 15)
         {
            if (Math.abs(angle) > 90)
            {
               if (angle < 0) turn(-90);
               else turn (90);
            }
            else turn(angle);
         }
      }
      else
      {
         if (Math.abs(ballangle) > 15)
         {
            if (Math.abs(ballangle) > 90)
            {
               if (ballangle < 0) turn(-90);
               else turn (90);
            }
            else turn(ballangle);
         }
      }
   }

   /****************************************************************************
   *   public void runIntoField()
   *
   *   Falls sich ein Spieler auerhalb des Feldes befindet, soll er wieder
   *   rein laufen
   *   @author Bjrn Konrad
   *   @version 1.0
   ****************************************************************************/
   public void runIntoField()
   {
      boolean seheFlagge = false;
      for (int i=49;i<=55;i++)
      {
         if(wmodel.getFlagTime(i) == wmodel.getCurrentTime() && Math.abs(wmodel.getFlagDirection(i)) <= 15)
         {
            dash(100);
            seheFlagge = true;
         }
      }
      if (seheFlagge == false)  turn(15);
   }   //Ende runIntoField

}   // class TacticalMoves
